@@ -17,9 +17,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "config.h"
 
 #include <algorithm>
 #include <sstream>
@@ -31,13 +29,13 @@
 #include "emuopl.h"
 #include "silentopl.h"
 #include "players.h"
-#include "audacious/plugin.h"
-#include "audacious/util.h"
-#include "audacious/configdb.h"
 #include <audacious/i18n.h>
 extern "C"
 {
-#include "audacious/output.h"
+#include <audacious/plugin.h>
+#include <audacious/configdb.h>
+#include <audacious/output.h>
+#include <audacious/util.h>
 }
 
 
@@ -71,7 +69,7 @@ static struct
   gint freq;
   gboolean bit16, stereo, endless;
   CPlayers players;
-} cfg =
+} conf =
 {
 44100l, true, false, false, CAdPlug::players};
 
@@ -95,9 +93,6 @@ static struct
 
 static InputPlayback *playback;
 
-// XXX: this function is not in the public API yet.
-extern "C" VFSFile * vfs_buffered_file_new_from_uri (gchar * uri);
-
 /***** Debugging *****/
 
 #ifdef DEBUG
@@ -144,6 +139,7 @@ print_left (const gchar * text)
   return GTK_WIDGET (label);
 }
 
+#if 0
 static void
 MessageBox (const char *title, const char *text, const char *button)
 {
@@ -155,7 +151,7 @@ MessageBox (const char *title, const char *text, const char *button)
   strcpy (tmptxt, text);
   strcpy (tmpbutton, button);
 
-  GtkWidget *msgbox = xmms_show_message (tmptitle, tmptxt, tmpbutton, FALSE,
+  GtkWidget *msgbox = audacious_info_dialog (tmptitle, tmptxt, tmpbutton, FALSE,
                                          G_CALLBACK (gtk_widget_destroyed),
                                          &msgbox);
 
@@ -163,6 +159,7 @@ MessageBox (const char *title, const char *text, const char *button)
   free (tmptxt);
   free (tmpbutton);
 }
+#endif
 
 /***** Dialog boxes *****/
 
@@ -182,7 +179,7 @@ adplug_about (void)
                                     "Linked AdPlug library version: "),
                                    version_text, NULL);
     about_win =
-      xmms_show_message (about_title, about_text, _("Ok"), FALSE, NULL, NULL);
+      audacious_info_dialog (about_title, about_text, _("Ok"), FALSE, NULL, NULL);
     g_signal_connect (G_OBJECT (about_win), "destroy",
                       G_CALLBACK (gtk_widget_destroyed), &about_win);
     g_free (about_text);
@@ -195,31 +192,31 @@ static void
 close_config_box_ok (GtkButton * button, GPtrArray * rblist)
 {
   // Apply configuration settings
-  cfg.bit16 =
+  conf.bit16 =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                                   (g_ptr_array_index (rblist, 0)));
-  cfg.stereo =
+  conf.stereo =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                                   (g_ptr_array_index (rblist, 1)));
 
   if (gtk_toggle_button_get_active
       (GTK_TOGGLE_BUTTON (g_ptr_array_index (rblist, 2))))
-    cfg.freq = 11025;
+    conf.freq = 11025;
   if (gtk_toggle_button_get_active
       (GTK_TOGGLE_BUTTON (g_ptr_array_index (rblist, 3))))
-    cfg.freq = 22050;
+    conf.freq = 22050;
   if (gtk_toggle_button_get_active
       (GTK_TOGGLE_BUTTON (g_ptr_array_index (rblist, 4))))
-    cfg.freq = 44100;
+    conf.freq = 44100;
   if (gtk_toggle_button_get_active
       (GTK_TOGGLE_BUTTON (g_ptr_array_index (rblist, 5))))
-    cfg.freq = 48000;
+    conf.freq = 48000;
 
-  cfg.endless =
+  conf.endless =
     !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                                    (g_ptr_array_index (rblist, 6)));
 
-  cfg.players = *(CPlayers *) g_ptr_array_index (rblist, 7);
+  conf.players = *(CPlayers *) g_ptr_array_index (rblist, 7);
   delete (CPlayers *) g_ptr_array_index (rblist, 7);
 
   g_ptr_array_free (rblist, FALSE);
@@ -256,7 +253,7 @@ adplug_config (void)
   GtkTooltips *tooltips = gtk_tooltips_new ();
   GPtrArray *rblist = g_ptr_array_new ();
 
-  gtk_window_set_title (GTK_WINDOW (config_dlg), "AdPlug :: Configuration");
+  gtk_window_set_title (GTK_WINDOW (config_dlg), _("AdPlug :: Configuration"));
   gtk_window_set_policy (GTK_WINDOW (config_dlg), FALSE, FALSE, TRUE);  // Window is auto sized
   gtk_window_set_modal (GTK_WINDOW (config_dlg), TRUE);
   gtk_container_add (GTK_CONTAINER (config_dlg->vbox), GTK_WIDGET (notebook));
@@ -265,7 +262,7 @@ adplug_config (void)
   {
     GtkWidget *button;
 
-    button = gtk_button_new_with_label ("Ok");
+    button = gtk_button_new_with_label (_("Ok"));
     g_signal_connect (G_OBJECT (button), "clicked",
                       G_CALLBACK (close_config_box_ok), (gpointer) rblist);
     g_signal_connect_data (G_OBJECT (button), "clicked",
@@ -275,7 +272,7 @@ adplug_config (void)
                                             G_CONNECT_SWAPPED));
     gtk_container_add (GTK_CONTAINER (config_dlg->action_area), button);
 
-    button = gtk_button_new_with_label ("Cancel");
+    button = gtk_button_new_with_label (_("Cancel"));
     g_signal_connect (G_OBJECT (button), "clicked",
                       G_CALLBACK (close_config_box_cancel),
                       (gpointer) rblist);
@@ -291,7 +288,7 @@ adplug_config (void)
   gtk_table_set_row_spacings (table, 5);
   gtk_table_set_col_spacings (table, 5);
   gtk_notebook_append_page (notebook, GTK_WIDGET (table),
-                            print_left ("General"));
+                            print_left (_("General")));
 
   // Add "Sound quality" section
   {
@@ -303,70 +300,70 @@ adplug_config (void)
     gtk_table_set_col_spacings (sqt, 5);
     gtk_table_attach_defaults (table,
                                make_framed (GTK_WIDGET (sqt),
-                                            "Sound quality"), 0, 1, 0, 1);
+                                            _("Sound quality")), 0, 1, 0, 1);
 
     // Add "Resolution" section
     fvb = GTK_VBOX (gtk_vbox_new (TRUE, 0));
     gtk_table_attach_defaults (sqt,
-                               make_framed (GTK_WIDGET (fvb), "Resolution"),
+                               make_framed (GTK_WIDGET (fvb), _("Resolution")),
                                0, 1, 0, 1);
-    rb = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (NULL, "8bit"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), !cfg.bit16);
+    rb = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (NULL, _("8bit")));
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), !conf.bit16);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     rb =
       GTK_RADIO_BUTTON (gtk_radio_button_new_with_label_from_widget
-                        (rb, "16bit"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), cfg.bit16);
+                        (rb, _("16bit")));
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), conf.bit16);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     g_ptr_array_add (rblist, (gpointer) rb);
 
     // Add "Channels" section
     fvb = GTK_VBOX (gtk_vbox_new (TRUE, 0));
     gtk_table_attach_defaults (sqt,
-                               make_framed (GTK_WIDGET (fvb), "Channels"), 0,
+                               make_framed (GTK_WIDGET (fvb), _("Channels")), 0,
                                1, 1, 2);
-    rb = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (NULL, "Mono"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), !cfg.stereo);
+    rb = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (NULL, _("Mono")));
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), !conf.stereo);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     rb =
       GTK_RADIO_BUTTON (gtk_radio_button_new_with_label_from_widget
-                        (rb, "Stereo"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), cfg.stereo);
+                        (rb, _("Stereo")));
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), conf.stereo);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     gtk_tooltips_set_tip (tooltips, GTK_WIDGET (rb),
-                          "Setting stereo is not recommended, unless you need to. "
+                          _("Setting stereo is not recommended, unless you need to. "
                           "This won't add any stereo effects to the sound - OPL2 "
-                          "is just mono - but eats up more CPU power!", NULL);
+                          "is just mono - but eats up more CPU power!"), NULL);
     g_ptr_array_add (rblist, (gpointer) rb);
 
     // Add "Frequency" section
     fvb = GTK_VBOX (gtk_vbox_new (TRUE, 0));
     gtk_table_attach_defaults (sqt,
-                               make_framed (GTK_WIDGET (fvb), "Frequency"), 1,
+                               make_framed (GTK_WIDGET (fvb), _("Frequency")), 1,
                                2, 0, 2);
     rb = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (NULL, "11025"));
-    if (cfg.freq == 11025)
+    if (conf.freq == 11025)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), TRUE);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     g_ptr_array_add (rblist, (gpointer) rb);
     rb =
       GTK_RADIO_BUTTON (gtk_radio_button_new_with_label_from_widget
                         (rb, "22050"));
-    if (cfg.freq == 22050)
+    if (conf.freq == 22050)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), TRUE);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     g_ptr_array_add (rblist, (gpointer) rb);
     rb =
       GTK_RADIO_BUTTON (gtk_radio_button_new_with_label_from_widget
                         (rb, "44100"));
-    if (cfg.freq == 44100)
+    if (conf.freq == 44100)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), TRUE);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     g_ptr_array_add (rblist, (gpointer) rb);
     rb =
       GTK_RADIO_BUTTON (gtk_radio_button_new_with_label_from_widget
                         (rb, "48000"));
-    if (cfg.freq == 48000)
+    if (conf.freq == 48000)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb), TRUE);
     gtk_container_add (GTK_CONTAINER (fvb), GTK_WIDGET (rb));
     g_ptr_array_add (rblist, (gpointer) rb);
@@ -378,18 +375,18 @@ adplug_config (void)
     GtkCheckButton *cb;
 
     gtk_table_attach_defaults (table,
-                               make_framed (GTK_WIDGET (vb), "Playback"), 1,
+                               make_framed (GTK_WIDGET (vb), _("Playback")), 1,
                                2, 0, 1);
 
     cb =
-      GTK_CHECK_BUTTON (gtk_check_button_new_with_label ("Detect songend"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb), !cfg.endless);
+      GTK_CHECK_BUTTON (gtk_check_button_new_with_label (_("Detect songend")));
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb), !conf.endless);
     gtk_container_add (GTK_CONTAINER (vb), GTK_WIDGET (cb));
     gtk_tooltips_set_tip (tooltips, GTK_WIDGET (cb),
-                          "If enabled, XMMS will detect a song's ending, stop "
+                          _("If enabled, XMMS will detect a song's ending, stop "
                           "it and advance in the playlist. If disabled, XMMS "
                           "won't take notice of a song's ending and loop it all "
-                          "over again and again.", NULL);
+                          "over again and again."), NULL);
     g_ptr_array_add (rblist, (gpointer) cb);
   }
 
@@ -397,17 +394,17 @@ adplug_config (void)
 
   table = GTK_TABLE (gtk_table_new (1, 1, TRUE));
   gtk_notebook_append_page (notebook, GTK_WIDGET (table),
-                            print_left ("Formats"));
+                            print_left (_("Formats")));
 
   // Add "Format selection" section
   {
     GtkHBox *vb = GTK_HBOX (gtk_hbox_new (FALSE, 0));
     gtk_table_attach_defaults (table,
                                make_framed (GTK_WIDGET (vb),
-                                            "Format selection"), 0, 1, 0, 1);
+                                            _("Format selection")), 0, 1, 0, 1);
     // Add scrollable list
     {
-      const gchar *rowstr[] = { "Format", "Extension" };
+      const gchar *rowstr[] = { _("Format"), _("Extension") };
       GtkEventBox *eventbox = GTK_EVENT_BOX (gtk_event_box_new ());
       GtkScrolledWindow *formatswnd =
         GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
@@ -430,15 +427,15 @@ adplug_config (void)
         g_free (rws[0]);
         g_free (rws[1]);
         gtk_clist_set_row_data (fl, rownum, (gpointer) (*i));
-        if (find (cfg.players.begin (), cfg.players.end (), *i) !=
-            cfg.players.end ())
+        if (find (conf.players.begin (), conf.players.end (), *i) !=
+            conf.players.end ())
           gtk_clist_select_row (fl, rownum, 0);
       }
 
       gtk_clist_columns_autosize (fl);
       gtk_scrolled_window_set_policy (formatswnd, GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
-      gpointer pl = (gpointer) new CPlayers (cfg.players);
+      gpointer pl = (gpointer) new CPlayers (conf.players);
       g_signal_connect (G_OBJECT (fl), "select-row",
                         G_CALLBACK (config_fl_row_select), pl);
       g_signal_connect (G_OBJECT (fl), "unselect-row",
@@ -447,10 +444,10 @@ adplug_config (void)
       gtk_container_add (GTK_CONTAINER (eventbox), GTK_WIDGET (formatswnd));
       gtk_container_add (GTK_CONTAINER (vb), GTK_WIDGET (eventbox));
       gtk_tooltips_set_tip (tooltips, GTK_WIDGET (eventbox),
-                            "Selected file types will be recognized and played "
+                            _("Selected file types will be recognized and played "
                             "back by this plugin. Deselected types will be "
                             "ignored to make room for other plugins to play "
-                            "these files.", NULL);
+                            "these files."), NULL);
       g_ptr_array_add (rblist, pl);
     }
   }
@@ -478,7 +475,7 @@ factory (VFSFile * fd, Copl * newopl)
 
   dbg_printf ("factory(%p<%s>,opl): ", fd,
               fd->uri != NULL ? fd->uri : "unknown");
-  return CAdPlug::factory (fd, newopl, cfg.players);
+  return CAdPlug::factory (fd, newopl, conf.players);
 }
 
 static void adplug_stop (InputPlayback * data);
@@ -510,7 +507,7 @@ static void
 adplug_info_box (char *filename)
 {
   CSilentopl tmpopl;
-  VFSFile *fd = vfs_buffered_file_new_from_uri (filename);
+  VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
 
   if (!fd)
     return;
@@ -526,14 +523,14 @@ adplug_info_box (char *filename)
   std::ostringstream infotext;
   unsigned int i;
   GtkDialog *infobox = GTK_DIALOG (gtk_dialog_new ());
-  GtkButton *okay_button = GTK_BUTTON (gtk_button_new_with_label ("Ok"));
+  GtkButton *okay_button = GTK_BUTTON (gtk_button_new_with_label (_("Ok")));
 
   GtkVBox *box = GTK_VBOX (gtk_vbox_new (TRUE, 2));
   GtkHBox *hbox = GTK_HBOX (gtk_hbox_new (TRUE, 2));
   GtkHBox *hbox2 = GTK_HBOX (gtk_hbox_new (TRUE, 2));
 
   // Build file info box
-  gtk_window_set_title (GTK_WINDOW (infobox), "AdPlug :: File Info");
+  gtk_window_set_title (GTK_WINDOW (infobox), _("AdPlug :: File Info"));
   gtk_window_set_policy (GTK_WINDOW (infobox), FALSE, FALSE, TRUE); // Window is auto sized
 
   gtk_container_add (GTK_CONTAINER (infobox->vbox), GTK_WIDGET (box));
@@ -555,25 +552,25 @@ adplug_info_box (char *filename)
 //            GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
   gtk_box_pack_end (GTK_BOX (box), GTK_WIDGET (hbox2), TRUE, TRUE, 2);
   gtk_box_pack_end (GTK_BOX (box),
-                    make_framed (print_left (filename), "Filename"), TRUE,
+                    make_framed (print_left (filename), _("Filename")), TRUE,
                     TRUE, 2);
 
   // Add "Song info" section
-  infotext << "Title: " << p->gettitle () << std::endl <<
-    "Author: " << p->getauthor () << std::endl <<
-    "File Type: " << p->gettype () << std::endl <<
-    "Subsongs: " << p->getsubsongs () << std::endl <<
-    "Instruments: " << p->getinstruments ();
+  infotext << _("Title: ") << p->gettitle () << std::endl <<
+    _("Author: ") << p->getauthor () << std::endl <<
+    _("File Type: ") << p->gettype () << std::endl <<
+    _("Subsongs: ") << p->getsubsongs () << std::endl <<
+    _("Instruments: ") << p->getinstruments ();
   if (plr.p == p)
     infotext << std::ends;
   else
   {
-    infotext << std::endl << "Orders: " << p->getorders () << std::endl <<
-      "Patterns: " << p->getpatterns () << std::ends;
+    infotext << std::endl << _("Orders: ") << p->getorders () << std::endl <<
+      _("Patterns: ") << p->getpatterns () << std::ends;
   }
   gtk_container_add (GTK_CONTAINER (hbox),
                      make_framed (print_left (infotext.str ().c_str ()),
-                                  "Song"));
+                                  _("Song")));
 
   // Add "Playback info" section if currently playing
   if (plr.p == p)
@@ -582,7 +579,7 @@ adplug_info_box (char *filename)
     gtk_label_set_justify (plr.infobox, GTK_JUSTIFY_LEFT);
     gtk_misc_set_padding (GTK_MISC (plr.infobox), 2, 2);
     gtk_container_add (GTK_CONTAINER (hbox),
-                       make_framed (GTK_WIDGET (plr.infobox), "Playback"));
+                       make_framed (GTK_WIDGET (plr.infobox), _("Playback")));
   }
 
 // Former packer layout, for future reproduction
@@ -599,7 +596,7 @@ adplug_info_box (char *filename)
     gchar tmpstr[10];
 
     {
-      const gchar *rowstr[] = { "#", "Instrument name" };
+      const gchar *rowstr[] = { "#", _("Instrument name") };
       instnames = GTK_CLIST (gtk_clist_new_with_titles (2, (gchar **)rowstr));
     }
     gtk_clist_set_column_justification (instnames, 0, GTK_JUSTIFY_RIGHT);
@@ -641,7 +638,7 @@ adplug_info_box (char *filename)
 //    gtk_packer_add(packer, make_framed(GTK_WIDGET(msgwnd), "Song message"),
 //         GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X, 2, 0, 0, 200, 50);
     gtk_box_pack_end (GTK_BOX (hbox2),
-                      make_framed (GTK_WIDGET (msgwnd), "Song message"), TRUE,
+                      make_framed (GTK_WIDGET (msgwnd), _("Song message")), TRUE,
                       TRUE, 2);
   }
 
@@ -663,7 +660,7 @@ adplug_info_box (char *filename)
 //   gtk_packer_add_defaults(packer, make_framed(GTK_WIDGET(slider), "Subsong selection"),
 //              GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
     gtk_box_pack_end (GTK_BOX (hbox2),
-                      make_framed (GTK_WIDGET (slider), "Subsong selection"),
+                      make_framed (GTK_WIDGET (slider), _("Subsong selection")),
                       TRUE, TRUE, 2);
   }
 
@@ -679,19 +676,61 @@ adplug_info_box (char *filename)
 
 /***** Main player (!! threaded !!) *****/
 
+static Tuple*
+adplug_get_tuple (char *filename)
+{
+  CSilentopl tmpopl;
+  VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
+
+  if (!fd)
+    return NULL;
+
+  CPlayer *p = factory (fd, &tmpopl);
+
+  if (p)
+  {
+    Tuple *ti = aud_tuple_new_from_filename(filename);
+    if (! p->getauthor().empty())
+      aud_tuple_associate_string(ti, FIELD_ARTIST, NULL, p->getauthor().c_str());
+    if (! p->gettitle().empty())
+      aud_tuple_associate_string(ti, FIELD_TITLE, NULL, p->gettitle().c_str());
+    else if (! p->getdesc().empty())
+      aud_tuple_associate_string(ti, FIELD_TITLE, NULL, p->getdesc().c_str());
+    else
+      aud_tuple_associate_string(ti, FIELD_TITLE, NULL, g_path_get_basename(filename));
+    aud_tuple_associate_string(ti, FIELD_CODEC, NULL, p->gettype().c_str());
+    aud_tuple_associate_string(ti, FIELD_QUALITY, NULL, "sequenced");
+    aud_tuple_associate_int(ti, FIELD_LENGTH, NULL, p->songlength (plr.subsong));
+    delete p;
+    return ti;
+  }
+
+  return NULL;
+}
+
+static char* format_and_free_ti( Tuple* ti, int* length )
+{
+  char* result = aud_tuple_formatter_make_title_string(ti, aud_get_gentitle_format());
+  if ( result )
+    *length = aud_tuple_get_int(ti, FIELD_LENGTH, NULL);
+  aud_tuple_free((void *) ti);
+
+  return result;
+}
+
 static void
 update_infobox (void)
 {
   std::ostringstream infotext;
 
   // Recreate info string
-  infotext << "Order: " << plr.p->getorder () << " / " << plr.p->
-    getorders () << std::endl << "Pattern: " << plr.p->
+  infotext << _("Order: ") << plr.p->getorder () << " / " << plr.p->
+    getorders () << std::endl << _("Pattern: ") << plr.p->
     getpattern () << " / " << plr.p->
-    getpatterns () << std::endl << "Row: " << plr.p->
-    getrow () << std::endl << "Speed: " << plr.p->
-    getspeed () << std::endl << "Timer: " << plr.p->
-    getrefresh () << "Hz" << std::ends;
+    getpatterns () << std::endl << _("Row: ") << plr.p->
+    getrow () << std::endl << _("Speed: ") << plr.p->
+    getspeed () << std::endl << _("Timer: ") << plr.p->
+    getrefresh () << _("Hz") << std::ends;
 
   GDK_THREADS_ENTER ();
   gtk_label_set_text (plr.infobox, infotext.str ().c_str ());
@@ -708,23 +747,24 @@ play_loop (void *data)
   InputPlayback *playback = (InputPlayback *) data;
   char *filename = (char *) playback->filename;
   dbg_printf ("play_loop(\"%s\"): ", filename);
-  CEmuopl opl (cfg.freq, cfg.bit16, cfg.stereo);
+  CEmuopl opl (conf.freq, conf.bit16, conf.stereo);
   long toadd = 0, i, towrite;
   char *sndbuf, *sndbufpos;
+  int songlength;
   bool playing = true,          // Song self-end indicator.
-    bit16 = cfg.bit16,          // Duplicate config, so it doesn't affect us if
-    stereo = cfg.stereo;        // the user changes it while we're playing.
-  unsigned long freq = cfg.freq;
+    bit16 = conf.bit16,          // Duplicate config, so it doesn't affect us if
+    stereo = conf.stereo;        // the user changes it while we're playing.
+  unsigned long freq = conf.freq;
 
   // we use VfsBufferedFile class here because adplug does a lot of
   // probing. a short delay before probing begins is better than
   // a lot of delay during probing.
-  VFSFile *fd = vfs_buffered_file_new_from_uri (playback->filename);
+  VFSFile *fd = aud_vfs_buffered_file_new_from_uri (playback->filename);
 
   if (!fd)
   {
     plr.playing = false;
-    g_thread_exit (NULL);
+    return (NULL);
   }
 
   // Try to load module
@@ -734,20 +774,17 @@ play_loop (void *data)
     dbg_printf ("error!\n");
     // MessageBox("AdPlug :: Error", "File could not be opened!", "Ok");
     plr.playing = false;
-    g_thread_exit (NULL);
+    return (NULL);
   }
 
-  // Cache song length
-  dbg_printf ("length, ");
-  plr.songlength = plr.p->songlength (plr.subsong);
-
-  // cache song title
+  // cache song title & length from tuple
   dbg_printf ("title, ");
-  if (!plr.p->gettitle ().empty ())
+  Tuple* ti = adplug_get_tuple(filename);
+  if (ti)
   {
-    plr.songtitle = (char *) malloc (plr.p->gettitle ().length () + 1);
-    strcpy (plr.songtitle, plr.p->gettitle ().c_str ());
+    plr.songtitle = format_and_free_ti( ti, &songlength );
   }
+  plr.songlength = songlength;
 
   // reset to first subsong on new file
   dbg_printf ("subsong, ");
@@ -763,7 +800,7 @@ play_loop (void *data)
 
   // Set XMMS main window information
   dbg_printf ("xmms, ");
-  adplug_ip.set_info (plr.songtitle, plr.songlength, freq * sampsize * 8,
+  playback->set_params (playback, plr.songtitle, plr.songlength, freq * sampsize * 8,
                       freq, stereo ? 2 : 1);
 
   // Rewind player to right subsong
@@ -772,7 +809,7 @@ play_loop (void *data)
 
   // main playback loop
   dbg_printf ("loop.\n");
-  while ((playing || cfg.endless) && plr.playing)
+  while ((playing || conf.endless) && plr.playing)
   {
     // seek requested ?
     if (plr.seek != -1)
@@ -812,9 +849,7 @@ play_loop (void *data)
     }
 
     // write sound buffer
-    while (playback->output->buffer_free () < SNDBUFSIZE * sampsize)
-      xmms_usleep (10000);
-    produce_audio (playback->output->written_time (),
+    playback->pass_audio(playback,
                    bit16 ? FORMAT_16 : FORMAT_8,
                    stereo ? 2 : 1, SNDBUFSIZE * sampsize, sndbuf, NULL);
 
@@ -829,7 +864,7 @@ play_loop (void *data)
   {                             // wait for output plugin to finish if song has self-ended
     dbg_printf ("wait, ");
     while (playback->output->buffer_playing ())
-      xmms_usleep (10000);
+      g_usleep (10000);
   }
   else
   {                             // or else, flush its output buffers
@@ -850,8 +885,7 @@ play_loop (void *data)
   free (sndbuf);
   plr.playing = false;          // important! XMMS won't get a self-ended song without it.
   dbg_printf (".\n");
-  vfs_fclose (fd);
-  g_thread_exit (NULL);
+  aud_vfs_fclose (fd);
   return (NULL);
 }
 
@@ -884,7 +918,7 @@ static int
 adplug_is_our_file (char *filename)
 {
   CSilentopl tmpopl;
-  VFSFile *fd = vfs_buffered_file_new_from_uri (filename);
+  VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
   if (!fd)
     return FALSE;
 
@@ -922,36 +956,12 @@ adplug_get_time (InputPlayback * data)
 static void
 adplug_song_info (char *filename, char **title, int *length)
 {
-  CSilentopl tmpopl;
-  VFSFile *fd = vfs_buffered_file_new_from_uri (filename);
+  *length = -1;
+  *title = NULL;
 
-  if (!fd)
-    return;
-
-  CPlayer *p = factory (fd, &tmpopl);
-
-  dbg_printf ("adplug_song_info(\"%s\", \"%s\", %d): ", filename, *title,
-              *length);
-
-  if (p)
-  {
-    // allocate and set title string
-    if (p->gettitle ().empty ())
-      *title = 0;
-    else
-    {
-      *title = (char *) malloc (p->gettitle ().length () + 1);
-      strcpy (*title, p->gettitle ().c_str ());
-    }
-
-    // get song length
-    *length = p->songlength (plr.subsong);
-
-    // delete temporary player object
-    delete p;
-  }
-
-  dbg_printf ("title = \"%s\", length = %d\n", *title, *length);
+  Tuple* ti = adplug_get_tuple( filename );
+  if ( ti )
+    *title = format_and_free_ti( ti, length );
 }
 
 /***** Player control *****/
@@ -972,8 +982,8 @@ adplug_play (InputPlayback * data)
   // open output plugin
   dbg_printf ("open, ");
   if (!playback->output->
-      open_audio (cfg.bit16 ? FORMAT_16 : FORMAT_8, cfg.freq,
-                  cfg.stereo ? 2 : 1))
+      open_audio (conf.bit16 ? FORMAT_16 : FORMAT_8, conf.freq,
+                  conf.stereo ? 2 : 1))
   {
     audio_error = TRUE;
     return;
@@ -987,9 +997,11 @@ adplug_play (InputPlayback * data)
   plr.time_ms = 0.0f;
   plr.seek = -1;
 
-  // start player thread
-  dbg_printf ("create");
-  plr.play_thread = g_thread_create (play_loop, playback, TRUE, NULL);
+  // start player func
+  dbg_printf ("play");
+  plr.play_thread =  g_thread_self();
+  playback->set_pb_ready(playback);
+  play_loop(playback);
   dbg_printf (".\n");
 }
 
@@ -1026,15 +1038,15 @@ static void
 adplug_init (void)
 {
   dbg_printf ("adplug_init(): open, ");
-  ConfigDb *db = bmp_cfg_db_open ();
+  ConfigDb *db = aud_cfg_db_open ();
 
   // Read configuration
   dbg_printf ("read, ");
-  bmp_cfg_db_get_bool (db, CFG_VERSION, "16bit", (gboolean *) & cfg.bit16);
-  bmp_cfg_db_get_bool (db, CFG_VERSION, "Stereo", (gboolean *) & cfg.stereo);
-  bmp_cfg_db_get_int (db, CFG_VERSION, "Frequency", (gint *) & cfg.freq);
-  bmp_cfg_db_get_bool (db, CFG_VERSION, "Endless",
-                       (gboolean *) & cfg.endless);
+  aud_cfg_db_get_bool (db, CFG_VERSION, "16bit", (gboolean *) & conf.bit16);
+  aud_cfg_db_get_bool (db, CFG_VERSION, "Stereo", (gboolean *) & conf.stereo);
+  aud_cfg_db_get_int (db, CFG_VERSION, "Frequency", (gint *) & conf.freq);
+  aud_cfg_db_get_bool (db, CFG_VERSION, "Endless",
+                       (gboolean *) & conf.endless);
 
   // Read file type exclusion list
   dbg_printf ("exclusion, ");
@@ -1042,18 +1054,18 @@ adplug_init (void)
     gchar *cfgstr = NULL, *exclude = NULL;
     gboolean cfgread;
 
-    cfgread = bmp_cfg_db_get_string (db, CFG_VERSION, "Exclude", &cfgstr);
+    cfgread = aud_cfg_db_get_string (db, CFG_VERSION, "Exclude", &cfgstr);
     if (cfgread) {
         exclude = (char *) malloc (strlen (cfgstr) + 2);
         strcpy (exclude, cfgstr);
         exclude[strlen (exclude) + 1] = '\0';
         g_strdelimit (exclude, ":", '\0');
         for (gchar * p = exclude; *p; p += strlen (p) + 1)
-            cfg.players.remove (cfg.players.lookup_filetype (p));
+            conf.players.remove (conf.players.lookup_filetype (p));
         free (exclude); free (cfgstr);
     }
   }
-  bmp_cfg_db_close (db);
+  aud_cfg_db_close (db);
 
   // Load database from disk and hand it to AdPlug
   dbg_printf ("database");
@@ -1064,15 +1076,10 @@ adplug_init (void)
 
     if (homedir)
     {
-      char *userdb =
-        (char *) malloc (strlen (homedir) + strlen (ADPLUG_CONFDIR) +
-                         strlen (ADPLUGDB_FILE) + 3);
-      strcpy (userdb, homedir);
-      strcat (userdb, "/" ADPLUG_CONFDIR "/");
-      strcat (userdb, ADPLUGDB_FILE);
+      std::string userdb;
+      userdb = "file://" + std::string(g_get_home_dir()) + "/" ADPLUG_CONFDIR "/" + ADPLUGDB_FILE;
       plr.db->load (userdb);    // load user's database
-      dbg_printf (" (userdb=\"%s\")", userdb);
-      free (userdb);
+      dbg_printf (" (userdb=\"%s\")", userdb.c_str());
     }
   }
   CAdPlug::set_database (plr.db);
@@ -1083,7 +1090,7 @@ static void
 adplug_quit (void)
 {
   dbg_printf ("adplug_quit(): open, ");
-  ConfigDb *db = bmp_cfg_db_open ();
+  ConfigDb *db = aud_cfg_db_open ();
 
   // Close database
   dbg_printf ("db, ");
@@ -1092,28 +1099,28 @@ adplug_quit (void)
 
   // Write configuration
   dbg_printf ("write, ");
-  bmp_cfg_db_set_bool (db, CFG_VERSION, "16bit", cfg.bit16);
-  bmp_cfg_db_set_bool (db, CFG_VERSION, "Stereo", cfg.stereo);
-  bmp_cfg_db_set_int (db, CFG_VERSION, "Frequency", cfg.freq);
-  bmp_cfg_db_set_bool (db, CFG_VERSION, "Endless", cfg.endless);
+  aud_cfg_db_set_bool (db, CFG_VERSION, "16bit", conf.bit16);
+  aud_cfg_db_set_bool (db, CFG_VERSION, "Stereo", conf.stereo);
+  aud_cfg_db_set_int (db, CFG_VERSION, "Frequency", conf.freq);
+  aud_cfg_db_set_bool (db, CFG_VERSION, "Endless", conf.endless);
 
   dbg_printf ("exclude, ");
   std::string exclude;
   for (CPlayers::const_iterator i = CAdPlug::players.begin ();
        i != CAdPlug::players.end (); i++)
-    if (find (cfg.players.begin (), cfg.players.end (), *i) ==
-        cfg.players.end ())
+    if (find (conf.players.begin (), conf.players.end (), *i) ==
+        conf.players.end ())
     {
       if (!exclude.empty ())
         exclude += ":";
       exclude += (*i)->filetype;
     }
   gchar *cfgval = g_strdup (exclude.c_str ());
-  bmp_cfg_db_set_string (db, CFG_VERSION, "Exclude", cfgval);
+  aud_cfg_db_set_string (db, CFG_VERSION, "Exclude", cfgval);
   free (cfgval);
 
   dbg_printf ("close");
-  bmp_cfg_db_close (db);
+  aud_cfg_db_close (db);
   dbg_printf (".\n");
 }
 
@@ -1124,8 +1131,10 @@ InputPlugin adplug_ip = {
   NULL,                         // filename (filled by XMMS)
   (gchar *)ADPLUG_NAME,                  // plugin description
   adplug_init,                  // plugin functions...
+  adplug_quit,
   adplug_about,
   adplug_config,
+  FALSE,
   adplug_is_our_file,
   NULL,                         // scan_dir (look in Input/cdaudio/cdaudio.c)
   adplug_play,
@@ -1136,23 +1145,21 @@ InputPlugin adplug_ip = {
   adplug_get_time,
   NULL,                         // get_volume (handled by output plugin)
   NULL,                         // set_volume (...)
-  adplug_quit,
   NULL,                         // OBSOLETE - DO NOT USE!
   NULL,                         // add_vis_pcm (filled by XMMS)
   NULL,                         // set_info (filled by XMMS)
   NULL,                         // set_info_text (filled by XMMS)
   adplug_song_info,
-  adplug_info_box,              // adplug_info_box was here (but it used deprecated GTK+ functions)
+  adplug_info_box,
   NULL,                         // output plugin (filled by XMMS)
-  NULL,
+  adplug_get_tuple,
   NULL,
   NULL,
   adplug_is_our_fd,
   NULL,
 };
 
-extern "C" InputPlugin *
-get_iplugin_info (void)
-{
-  return &adplug_ip;
-}
+InputPlugin *adplug_iplist[] = { &adplug_ip, NULL };
+
+DECLARE_PLUGIN(adplug, NULL, NULL, adplug_iplist, NULL, NULL, NULL, NULL,NULL);
+
