@@ -69,14 +69,14 @@ extern gboolean scan_file(struct mad_info_t *info, gboolean fast);
 /**
  * init the mad_info_t struct.
  */
-gboolean input_init(struct mad_info_t * info, const char *url)
+gboolean
+input_init(struct mad_info_t * info, const char *url, VFSFile *fd)
 {
-#ifdef DEBUG
-    g_message("f: input_init");
-#endif
-    memset(info, 0, sizeof(struct mad_info_t));
+    AUDDBG("f: input_init\n");
 
-    info->fmt = FMT_S16_LE;
+    memset(info, 0, sizeof(struct mad_info_t)); // all fields are cleared to 0 --yaz
+
+    info->fmt = FMT_FIXED32;
     info->channels = -1;
     info->mpeg_layer = -1;
     info->size = -1;
@@ -85,56 +85,45 @@ gboolean input_init(struct mad_info_t * info, const char *url)
     info->duration = mad_timer_zero;
     info->pos = mad_timer_zero;
     info->url = g_strdup(url);
-    info->current_frame = 0;
-    info->frames = 0;
-    info->bitrate = 0;
-    info->vbr = 0;
-    info->mode = 0;
-    info->title = 0;
-    info->offset = 0;
-    info->prev_title = NULL;
-
-    info->replaygain_album_str = 0;
-    info->replaygain_track_str = 0;
-    info->replaygain_album_peak_str = 0;
-    info->replaygain_track_peak_str = 0;
-    info->mp3gain_undo_str = 0;
-    info->mp3gain_minmax_str = 0;
+    info->filename = g_strdup(url);
 
     // from input_read_replaygain()
-    info->has_replaygain = FALSE;
-    info->replaygain_album_scale = -1;
-    info->replaygain_track_scale = -1;
+    info->replaygain_track_peak = 0.0;
+    info->replaygain_track_scale = 0.0;
+    info->replaygain_album_peak = 0.0;
+    info->replaygain_album_scale = 0.0;
     info->mp3gain_undo = -77;
     info->mp3gain_minmax = -77;
 
-    info->tuple = NULL;
-
-    info->filename = g_strdup(url);
-
-    info->infile = vfs_fopen(info->filename, "rb");
-    if (info->infile == NULL) {
-        return FALSE;
+    if(!fd){
+        info->infile = aud_vfs_fopen(info->filename, "rb");
+        if (info->infile == NULL) {
+            return FALSE;
+        }
     }
+    else{
+        AUDDBG("input_init: aud_vfs_dup\n");
+
+        info->infile = aud_vfs_dup(fd);
+    }
+
     // obtain file size
-    vfs_fseek(info->infile, 0, SEEK_END);
-    info->size = vfs_ftell(info->infile);
-    vfs_fseek(info->infile, 0, SEEK_SET);
+    info->size = aud_vfs_fsize(info->infile);
     info->remote = info->size == 0 ? TRUE : FALSE; //proxy connection may result in non-zero size.
-    if(audmad_is_remote((gchar *)url))
+    if(aud_vfs_is_remote((gchar *)url))
         info->remote = TRUE;
 
     info->fileinfo_request = FALSE;
 
-#ifdef DEBUG
-    g_message("i: info->size = %lu", (long unsigned int)info->size);
-    g_message("e: input_init");
-#endif
+    AUDDBG("i: info->size = %lu\n", (long unsigned int)info->size);
+    AUDDBG("e: input_init\n");
+
     return TRUE;
 }
 
 /* return length in letters */
-size_t mad_ucs4len(id3_ucs4_t *ucs)
+size_t
+mad_ucs4len(id3_ucs4_t *ucs)
 {
     id3_ucs4_t *ptr = ucs;
     size_t len = 0;
@@ -146,7 +135,8 @@ size_t mad_ucs4len(id3_ucs4_t *ucs)
 }
 
 /* duplicate id3_ucs4_t string. new string will be terminated with 0. */
-id3_ucs4_t *mad_ucs4dup(id3_ucs4_t *org)
+id3_ucs4_t *
+mad_ucs4dup(id3_ucs4_t *org)
 {
     id3_ucs4_t *new = NULL;
     size_t len = mad_ucs4len(org);
@@ -160,7 +150,8 @@ id3_ucs4_t *mad_ucs4dup(id3_ucs4_t *org)
 
 #define BYTES(x) ((x) * sizeof(id3_ucs4_t))
 
-id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
+id3_ucs4_t *
+mad_parse_genre(const id3_ucs4_t *string)
 {
     id3_ucs4_t *ret = NULL;
     id3_ucs4_t *tmp = NULL;
@@ -184,9 +175,9 @@ id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
         ret = g_malloc0(1024);
     }
 
-    for(ptr = (id3_ucs4_t *)string; *ptr != 0 && ptr <= tail; ptr++) {
+    for(ptr = (id3_ucs4_t *)string; ptr <= tail && *ptr != 0; ptr++) {
         if(*ptr == '(') {
-            if(*(++ptr) == '(') { // escaped text like: ((something)
+            if(ptr < tail && *(++ptr) == '(') { // escaped text like: ((something)
                 for(end = ptr; *end != ')' && *end != 0;) { // copy "(something)"
                     end++;
                 }
@@ -196,7 +187,7 @@ id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
                 *(ret + ret_len) = 0; //terminate
                 ptr = end + 1;
             }
-            else {
+            else if (ptr <= tail && *ptr != 0) {
                 // reference to an id3v1 genre code
                 for(end = ptr; *end != ')' && *end != 0;) {
                     end++;
@@ -214,7 +205,7 @@ id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
 
                 tmp_len = mad_ucs4len(genre);
 
-                memcpy(ret + BYTES(ret_len), genre, BYTES(tmp_len));
+                memcpy(ret + ret_len, genre, BYTES(tmp_len));
 
                 ret_len += tmp_len;
                 *(ret + ret_len) = 0; //terminate
@@ -234,36 +225,34 @@ id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
                 }
                 tp++;
             }
+
             if(is_num) {
-#ifdef DEBUG
-                printf("is_num!\n");
-#endif
+                AUDDBG("is_num!\n");
+
                 tmp = g_malloc0(BYTES(end - ptr + 1));
                 memcpy(tmp, ptr, BYTES(end - ptr));
                 *(tmp + (end - ptr)) = 0; //terminate
                 ptr += end - ptr;
 
                 genre = (id3_ucs4_t *)id3_genre_name((const id3_ucs4_t *)tmp);
-#ifdef DEBUG
-                printf("genre length = %d\n", mad_ucs4len(genre));
-#endif
+                AUDDBG("genre length = %d\n", mad_ucs4len(genre));
+
                 g_free(tmp);
                 tmp = NULL;
 
                 tmp_len = mad_ucs4len(genre);
 
-                memcpy(ret + BYTES(ret_len), genre, BYTES(tmp_len));
+                memcpy(ret + ret_len, genre, BYTES(tmp_len));
 
                 ret_len += tmp_len;
                 *(ret + ret_len) = 0; //terminate
             }
             else { // plain text
-#ifdef DEBUG
-                printf("plain!\n");
-                printf("ret_len = %d\n", ret_len);
-                printf("end - ptr = %d\n", BYTES(end - ptr));
-#endif
-                memcpy(ret + BYTES(ret_len), ptr, BYTES(end - ptr));
+                AUDDBG("plain!\n");
+                AUDDBG("ret_len = %d\n", ret_len);
+                AUDDBG("end - ptr = %d\n", BYTES(end - ptr));
+
+                memcpy(ret + ret_len, ptr, BYTES(end - ptr));
                 ret_len = ret_len + (end - ptr);
                 *(ret + ret_len) = 0; //terminate
                 ptr += (end - ptr);
@@ -273,7 +262,8 @@ id3_ucs4_t *mad_parse_genre(const id3_ucs4_t *string)
     return ret;
 }
 
-gchar *input_id3_get_string(struct id3_tag * tag, char *frame_name)
+gchar *
+input_id3_get_string(struct id3_tag * tag, const gchar *frame_name)
 {
     gchar *rtn0 = NULL, *rtn = NULL;
     const id3_ucs4_t *string_const = NULL;
@@ -318,7 +308,7 @@ gchar *input_id3_get_string(struct id3_tag * tag, char *frame_name)
     switch (encoding) {
     case ID3_FIELD_TEXTENCODING_ISO_8859_1:
         rtn0 = (gchar *)id3_ucs4_latin1duplicate(string);
-        rtn = str_to_utf8(rtn0);
+        rtn = aud_str_to_utf8(rtn0);
         g_free(rtn0);
         break;
     case ID3_FIELD_TEXTENCODING_UTF_8:
@@ -328,82 +318,84 @@ gchar *input_id3_get_string(struct id3_tag * tag, char *frame_name)
     }
     g_free((void *)string);
         
-#ifdef DEBUG
-    g_print("i: string = %s\n", rtn);
-#endif
+    AUDDBG("i: string = %s\n", rtn);
+
     return rtn;
 }
 
-
-static void input_alloc_tag(struct mad_info_t *info)
+static void
+input_set_and_free_tag(struct id3_tag *tag, Tuple *tuple, const gchar *frame, const gint nfield)
 {
-    TitleInput *title_input;
+    gchar *scratch = input_id3_get_string(tag, frame);
+
+    aud_tuple_associate_string(tuple, nfield, NULL, scratch);
+    aud_tuple_associate_string(tuple, -1, frame, scratch);
+
+    g_free(scratch);
+}
+
+static void
+input_alloc_tag(struct mad_info_t *info)
+{
+    Tuple *tuple;
 
     if (info->tuple == NULL) {
-        title_input = bmp_title_input_new();
-        info->tuple = title_input;
-        info->tuple->length = -1; //will be refferd in decoder.c
+        tuple = aud_tuple_new();
+        info->tuple = tuple;
+        aud_tuple_associate_int(info->tuple, FIELD_LENGTH, NULL, -1);
     }
-    else
-        title_input = info->tuple;
 }
 
 /**
  * read the ID3 tag 
  */
-static void input_read_tag(struct mad_info_t *info)
+static void
+input_read_tag(struct mad_info_t *info)
 {
     gchar *string = NULL;
-    TitleInput *title_input;
+    Tuple *tuple;
     glong curpos = 0;
 
-#ifdef DEBUG
-    g_message("f: input_read_tag");
-#endif
-    if (info->tuple == NULL) {
-        title_input = bmp_title_input_new();
-        info->tuple = title_input;
-    }
-    else
-        title_input = info->tuple;
+    AUDDBG("f: input_read_tag\n");
+
+    if (info->tuple != NULL)
+        aud_tuple_free(info->tuple);
+        
+    tuple = aud_tuple_new_from_filename(info->filename);
+    info->tuple = tuple;
 
     if(info->infile) {
-        curpos = vfs_ftell(info->infile);
+        curpos = aud_vfs_ftell(info->infile);
         info->id3file = id3_file_vfsopen(info->infile, ID3_FILE_MODE_READONLY);
     }
-    else
+    else {
         info->id3file = id3_file_open(info->filename, ID3_FILE_MODE_READONLY);
+    }
 
     if (!info->id3file) {
-#ifdef DEBUG
-        g_message("read_tag: no id3file");
-#endif
+        AUDDBG("read_tag: no id3file\n");
         return;
     }
 
     info->tag = id3_file_tag(info->id3file);
     if (!info->tag) {
-#ifdef DEBUG
-        g_message("read_tag: no tag");
-#endif
+        AUDDBG("read_tag: no tag\n");
         return;
     }
 
-    title_input->performer =
-        input_id3_get_string(info->tag, ID3_FRAME_ARTIST);
-    title_input->track_name =
-        input_id3_get_string(info->tag, ID3_FRAME_TITLE);
-    title_input->album_name =
-        input_id3_get_string(info->tag, ID3_FRAME_ALBUM);
-    title_input->genre = input_id3_get_string(info->tag, ID3_FRAME_GENRE);
-    title_input->comment =
-        input_id3_get_string(info->tag, ID3_FRAME_COMMENT);
+    input_set_and_free_tag(info->tag, tuple, ID3_FRAME_ARTIST, FIELD_ARTIST);
+    input_set_and_free_tag(info->tag, tuple, ID3_FRAME_TITLE, FIELD_TITLE);
+    input_set_and_free_tag(info->tag, tuple, ID3_FRAME_ALBUM, FIELD_ALBUM);
+    input_set_and_free_tag(info->tag, tuple, ID3_FRAME_GENRE, FIELD_GENRE);
+    input_set_and_free_tag(info->tag, tuple, ID3_FRAME_COMMENT, FIELD_COMMENT);
+
     string = input_id3_get_string(info->tag, ID3_FRAME_TRACK);
     if (string) {
-        title_input->track_number = atoi(string);
+        aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, atoi(string));
         g_free(string);
         string = NULL;
     }
+
     // year
     string = NULL;
     string = input_id3_get_string(info->tag, ID3_FRAME_YEAR);   //TDRC
@@ -411,88 +403,95 @@ static void input_read_tag(struct mad_info_t *info)
         string = input_id3_get_string(info->tag, "TYER");
 
     if (string) {
-        title_input->year = atoi(string);
+        aud_tuple_associate_int(tuple, FIELD_YEAR, NULL, atoi(string));
         g_free(string);
         string = NULL;
     }
 
     // length
-    title_input->length = -1;
     string = input_id3_get_string(info->tag, "TLEN");
-    if (string) {
-        title_input->length = atoi(string);
-#ifdef DEBUG
-        g_message("input_read_tag: TLEN = %d", title_input->length);
-#endif	
+    if (string && atoi(string)) {
+        aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, atoi(string));
+        AUDDBG("input_read_tag: TLEN = %d\n", atoi(string));
         g_free(string);
         string = NULL;
-    }
+    } else
+        aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, -1);
+    
+    aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "MPEG Audio (MP3)");
+    aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "lossy");
 
-    title_input->file_name = g_strdup(g_basename(info->filename));
-    title_input->file_path = g_path_get_dirname(info->filename);
-    if ((string = strrchr(title_input->file_name, '.'))) {
-        title_input->file_ext = string + 1;
-        *string = '\0';         // make filename end at dot.
-    }
-
-    info->title = xmms_get_titlestring(audmad_config.title_override == TRUE ?
-        audmad_config.id3_format : xmms_get_gentitle_format(), title_input);
+    info->title = aud_tuple_formatter_make_title_string(tuple, audmad_config->title_override == TRUE ?
+        audmad_config->id3_format : aud_get_gentitle_format());
 
     // for connection via proxy, we have to stop transfer once. I can't explain the reason.
     if (info->infile != NULL) {
-        vfs_fseek(info->infile, -1, SEEK_SET); // an impossible request
-        vfs_fseek(info->infile, curpos, SEEK_SET);
+        aud_vfs_fseek(info->infile, -1, SEEK_SET); // an impossible request
+        aud_vfs_fseek(info->infile, curpos, SEEK_SET);
     }
     
-#ifdef DEBUG
-    g_message("e: input_read_tag");
-#endif
-
+    AUDDBG("e: input_read_tag\n");
 }
 
-void input_process_remote_metadata(struct mad_info_t *info)
+void
+input_process_remote_metadata(struct mad_info_t *info)
 {
+    gboolean metadata = FALSE;
+
     if(info->remote && mad_timer_count(info->duration, MAD_UNITS_SECONDS) <= 0){
         gchar *tmp = NULL;
-#ifdef DEBUG
+
 #ifdef DEBUG_INTENSIVELY
-        g_message("process_remote_meta");
-#endif
+        AUDDBG("process_remote_meta\n");
 #endif
         g_free(info->title);
         info->title = NULL;
-        g_free(info->tuple->track_name);
-        info->tuple->track_name = NULL;
-        g_free(info->tuple->album_name);
-        info->tuple->album_name = NULL;
+        aud_tuple_disassociate(info->tuple, FIELD_TITLE, NULL);
+        aud_tuple_disassociate(info->tuple, FIELD_ALBUM, NULL);
 
-        tmp = vfs_get_metadata(info->infile, "track-name");
+        tmp = aud_vfs_get_metadata(info->infile, "track-name");
         if(tmp){
-            info->tuple->track_name = str_to_utf8(tmp);
-            info->title = g_strdup(info->tuple->track_name);
+            metadata = TRUE;
+            gchar *scratch;
+
+            scratch = aud_str_to_utf8(tmp);
+            aud_tuple_associate_string(info->tuple, FIELD_TITLE, NULL, scratch);
+            g_free(scratch);
+
             g_free(tmp);
             tmp = NULL;
         }
 
-        tmp = vfs_get_metadata(info->infile, "stream-name");
+        tmp = aud_vfs_get_metadata(info->infile, "stream-name");
         if(tmp){
-            info->tuple->album_name = str_to_utf8(tmp);
+            metadata = TRUE;
+            gchar *scratch;
+
+            scratch = aud_str_to_utf8(tmp);
+            aud_tuple_associate_string(info->tuple, FIELD_ALBUM, NULL, scratch);
+            aud_tuple_associate_string(info->tuple, -1, "stream", scratch);
+            g_free(scratch);
+
             g_free(tmp);
             tmp = NULL;
         }
 
-        if (info->tuple->track_name && info->tuple->album_name)
-            tmp = g_strdup_printf("%s (%s)", info->tuple->track_name, info->tuple->album_name);
-        else if (info->tuple->album_name)
-            tmp = g_strdup(info->tuple->album_name);
-        else
-            tmp = g_strdup(g_basename(info->filename));
+        if (metadata)
+            tmp = aud_tuple_formatter_process_string(info->tuple, "${?title:${title}}${?stream: (${stream})}");
+        else {
+            gchar *realfn = g_filename_from_uri(info->filename, NULL, NULL);
+            gchar *tmp2 = g_path_get_basename(realfn ? realfn : info->filename); // info->filename is uri. --yaz
+            tmp = aud_str_to_utf8(tmp2);
+            g_free(tmp2); tmp2 = NULL;
+            g_free(realfn); realfn = NULL;
+//            tmp = g_strdup(g_basename(info->filename)); //XXX maybe ok. --yaz
+        }
 
         /* call set_info only if tmp is different from prev_tmp */
         if ( ( ( info->prev_title != NULL ) && ( strcmp(info->prev_title,tmp) ) ) ||
              ( info->prev_title == NULL ) )
         {
-            mad_plugin->set_info(tmp,
+            info->playback->set_params(info->playback, tmp,
                                  -1, // indicate the stream is unseekable
                                  info->bitrate, info->freq, info->channels);
             if (info->prev_title)
@@ -509,11 +508,12 @@ void input_process_remote_metadata(struct mad_info_t *info)
  * Retrieve meta-information about URL.
  * For local files this means ID3 tag etc.
  */
-gboolean input_get_info(struct mad_info_t *info, gboolean fast_scan)
+gboolean
+input_get_info(struct mad_info_t *info, gboolean fast_scan)
 {
-#ifdef DEBUG
+#ifdef AUD_DEBUG
     gchar *tmp = g_filename_to_utf8(info->filename, -1, NULL, NULL, NULL);    
-    g_message("f: input_get_info: %s, fast_scan = %s", tmp, fast_scan ? "TRUE" : "FALSE");
+    AUDDBG("f: input_get_info: %s, fast_scan = %s\n", tmp, fast_scan ? "TRUE" : "FALSE");
     g_free(tmp);
 #endif                          /* DEBUG */
 
@@ -526,28 +526,24 @@ gboolean input_get_info(struct mad_info_t *info, gboolean fast_scan)
 
     /* scan mp3 file, decoding headers */
     if (scan_file(info, fast_scan) == FALSE) {
-#ifdef DEBUG
-        g_message("input_get_info: scan_file failed");
-#endif
+        AUDDBG("input_get_info: scan_file failed\n");
         return FALSE;
     }
 
     /* reset the input file to the start */
-    vfs_fseek(info->infile, 0, SEEK_SET);
+    aud_vfs_fseek(info->infile, 0, SEEK_SET);
     info->offset = 0;
 
     /* use the filename for the title as a last resort */
     if (!info->title) {
-        char *pos = strrchr(info->filename, DIR_SEPARATOR);
+        char *pos = strrchr(info->filename, DIR_SEPARATOR); //XXX info->filename is uri. --yaz
         if (pos)
             info->title = g_strdup(pos + 1);
         else
-            info->title = g_strdup(info->filename);
+            info->title = g_strdup(info->filename); //XXX info->filename is uri. --yaz
     }
 
-#ifdef DEBUG
-    g_message("e: input_get_info");
-#endif                          /* DEBUG */
+    AUDDBG("e: input_get_info\n");
     return TRUE;
 }
 
@@ -565,23 +561,20 @@ input_get_data(struct mad_info_t *info, guchar * buffer,
                int buffer_size)
 {
     int len = 0;
-#ifdef DEBUG
 #ifdef DEBUG_INTENSIVELY
-  g_message ("f: input_get_data: %d", buffer_size);
-#endif
+  AUDDBG ("f: input_get_data: %d\n", buffer_size);
 #endif
     /* simply read to data from the file */
-    len = vfs_fread(buffer, 1, buffer_size, info->infile); //vfs_fread returns num of elements.
+    len = aud_vfs_fread(buffer, 1, buffer_size, info->infile); //aud_vfs_fread returns num of elements.
 
     if(len == 0 && info->playback){
         info->playback->eof = TRUE;
     }
 
-#ifdef DEBUG
 #ifdef DEBUG_INTENSIVELY
-    g_message ("e: input_get_data: size=%d offset=%d", len, info->offset);
-#endif
+    AUDDBG ("e: input_get_data: size=%d offset=%d\n", len, info->offset);
 #endif
+
     info->offset += len;
     return len;
 }
@@ -589,11 +582,10 @@ input_get_data(struct mad_info_t *info, guchar * buffer,
 /**
  * Free up all mad_info_t related resourses.
  */
-gboolean input_term(struct mad_info_t * info)
+gboolean
+input_term(struct mad_info_t * info)
 {
-#ifdef DEBUG
-    g_message("f: input_term");
-#endif
+    AUDDBG("f: input_term\n");
 
     if (info->title)
         g_free(info->title);
@@ -602,7 +594,7 @@ gboolean input_term(struct mad_info_t * info)
     if (info->filename)
         g_free(info->filename);
     if (info->infile)
-        vfs_fclose(info->infile);
+        aud_vfs_fclose(info->infile);
     if (info->id3file)
         id3_file_close(info->id3file);
 
@@ -620,7 +612,7 @@ gboolean input_term(struct mad_info_t * info)
         g_free(info->mp3gain_minmax_str);
 
     if (info->tuple) {
-        bmp_title_input_free(info->tuple);
+        aud_tuple_free(info->tuple);
         info->tuple = NULL;
     }
 
@@ -629,8 +621,8 @@ gboolean input_term(struct mad_info_t * info)
 
     /* set everything to zero in case it gets used again. */
     memset(info, 0, sizeof(struct mad_info_t));
-#ifdef DEBUG
-    g_message("e: input_term");
-#endif
+
+    AUDDBG("e: input_term\n");
+
     return TRUE;
 }
