@@ -13,10 +13,20 @@
 #include "sndfile.h"
 #include "stddefs.h"
 #include "archive/open.h"
-#include "audacious/configdb.h"
-#include "audacious/vfs.h"
 extern "C" {
-#include "audacious/output.h"
+#include <audacious/configdb.h>
+#include <audacious/output.h>
+#include <audacious/strings.h>
+}
+
+static char* format_and_free_ti( Tuple* ti, int* length )
+{
+        char* result = aud_tuple_formatter_make_title_string(ti, aud_get_gentitle_format());
+        if ( result )
+                *length = aud_tuple_get_int(ti, FIELD_LENGTH, NULL);
+        aud_tuple_free((void *) ti);
+
+        return result;
 }
 
 // ModplugXMMS member functions ===============================
@@ -25,6 +35,7 @@ extern "C" {
 ModplugXMMS::ModplugXMMS()
 {
 	mSoundFile = new CSoundFile;
+	mOutPlug = NULL;
 }
 ModplugXMMS::~ModplugXMMS()
 {
@@ -65,45 +76,45 @@ void ModplugXMMS::Init(void)
 {
 	ConfigDb *db;
 	
-	db = bmp_cfg_db_open();
+	db = aud_cfg_db_open();
 
-	bmp_cfg_db_get_bool(db,"modplug","Surround", &mModProps.mSurround);
-        bmp_cfg_db_get_bool(db,"modplug","Oversampling", &mModProps.mOversamp);
-        bmp_cfg_db_get_bool(db,"modplug","Megabass", &mModProps.mMegabass);
-        bmp_cfg_db_get_bool(db,"modplug","NoiseReduction", &mModProps.mNoiseReduction);
-        bmp_cfg_db_get_bool(db,"modplug","VolumeRamp", &mModProps.mVolumeRamp);
-        bmp_cfg_db_get_bool(db,"modplug","Reverb", &mModProps.mReverb);
-        bmp_cfg_db_get_bool(db,"modplug","FastInfo", &mModProps.mFastinfo);
-        bmp_cfg_db_get_bool(db,"modplug","UseFileName", &mModProps.mUseFilename);
-        bmp_cfg_db_get_bool(db,"modplug","GrabAmigaMOD", &mModProps.mGrabAmigaMOD);
-        bmp_cfg_db_get_bool(db,"modplug","PreAmp", &mModProps.mPreamp);
-        bmp_cfg_db_get_float(db,"modplug","PreAmpLevel", &mModProps.mPreampLevel);
-        bmp_cfg_db_get_int(db,"modplug", "Channels", &mModProps.mChannels);
-        bmp_cfg_db_get_int(db,"modplug", "Bits", &mModProps.mBits);
-        bmp_cfg_db_get_int(db,"modplug", "Frequency", &mModProps.mFrequency);
-        bmp_cfg_db_get_int(db,"modplug", "ResamplineMode", &mModProps.mResamplingMode);
-        bmp_cfg_db_get_int(db,"modplug", "ReverbDepth", &mModProps.mReverbDepth);
-        bmp_cfg_db_get_int(db,"modplug", "ReverbDelay", &mModProps.mReverbDelay);
-        bmp_cfg_db_get_int(db,"modplug", "BassAmount", &mModProps.mBassAmount);
-        bmp_cfg_db_get_int(db,"modplug", "BassRange", &mModProps.mBassRange);
-        bmp_cfg_db_get_int(db,"modplug", "SurroundDepth", &mModProps.mSurroundDepth);
-        bmp_cfg_db_get_int(db,"modplug", "SurroundDelay", &mModProps.mSurroundDelay);
-        bmp_cfg_db_get_int(db,"modplug", "LoopCount", &mModProps.mLoopCount);
+	aud_cfg_db_get_bool(db, MODPLUG_CFGID,"Surround", &mModProps.mSurround);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"Oversampling", &mModProps.mOversamp);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"Megabass", &mModProps.mMegabass);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"NoiseReduction", &mModProps.mNoiseReduction);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"VolumeRamp", &mModProps.mVolumeRamp);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"Reverb", &mModProps.mReverb);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"FastInfo", &mModProps.mFastinfo);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"UseFileName", &mModProps.mUseFilename);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"GrabAmigaMOD", &mModProps.mGrabAmigaMOD);
+        aud_cfg_db_get_bool(db, MODPLUG_CFGID,"PreAmp", &mModProps.mPreamp);
+        aud_cfg_db_get_float(db, MODPLUG_CFGID,"PreAmpLevel", &mModProps.mPreampLevel);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "Channels", &mModProps.mChannels);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "Bits", &mModProps.mBits);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "Frequency", &mModProps.mFrequency);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "ResamplineMode", &mModProps.mResamplingMode);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "ReverbDepth", &mModProps.mReverbDepth);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "ReverbDelay", &mModProps.mReverbDelay);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "BassAmount", &mModProps.mBassAmount);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "BassRange", &mModProps.mBassRange);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "SurroundDepth", &mModProps.mSurroundDepth);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "SurroundDelay", &mModProps.mSurroundDelay);
+        aud_cfg_db_get_int(db, MODPLUG_CFGID, "LoopCount", &mModProps.mLoopCount);
 
-	bmp_cfg_db_close(db);
+	aud_cfg_db_close(db);
 }
 
 bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 {
 	string lExt;
 	uint32 lPos;
+	const int magicSize = 32;
+	char magic[magicSize];
 
-	gchar magic[4];
-
-	vfs_fread(magic, 1, 4, file);
+	aud_vfs_fread(magic, 1, magicSize, file);
 	if (!memcmp(magic, UMX_MAGIC, 4))
 		return true;
-	if (!memcmp(magic, XM_MAGIC, 4))
+	if (!memcmp(magic, "Extended Module:", 16))
 		return true;
 	if (!memcmp(magic, M669_MAGIC, 2))
 		return true;
@@ -114,16 +125,26 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 	if (!memcmp(magic, PSM_MAGIC, 4))
 		return true;
 
-	vfs_fseek(file, 44, SEEK_SET);
-	vfs_fread(magic, 1, 4, file);
+	aud_vfs_fseek(file, 44, SEEK_SET);
+	aud_vfs_fread(magic, 1, 4, file);
 	if (!memcmp(magic, S3M_MAGIC, 4))
 		return true;
-	vfs_fseek(file, 1080, SEEK_SET);
-	vfs_fread(magic, 1, 4, file);
-	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER6, 4))
-		return true;
-	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER8, 4))
-		return true;
+
+	aud_vfs_fseek(file, 1080, SEEK_SET);
+	aud_vfs_fread(magic, 1, 4, file);
+	
+	// Check for Fast Tracker multichannel modules (xCHN, xxCH)
+	if (magic[1] == 'C' && magic[2] == 'H' && magic[3] == 'N') {
+		if (magic[0] == '6' || magic[0] == '8')
+			return true;
+	}
+	if (magic[2] == 'C' && magic[3] == 'H' && isdigit(magic[0]) && isdigit(magic[1])) {
+		int nch = (magic[0] - '0') * 10 + (magic[1] - '0');
+		if ((nch % 2 == 0) && nch >= 10)
+			return true;
+	}
+	
+	// Check for Amiga MOD module formats
 	if(mModProps.mGrabAmigaMOD) {
 	if (!memcmp(magic, MOD_MAGIC_PROTRACKER4, 4))
 		return true;
@@ -221,13 +242,7 @@ bool ModplugXMMS::CanPlayFileFromVFS(const string& aFilename, VFSFile *file)
 	return false;
 }
 
-void* ModplugXMMS::PlayThread(void* arg)
-{
-	((ModplugXMMS*)arg)->PlayLoop();
-	return NULL;
-}
-
-void ModplugXMMS::PlayLoop()
+void ModplugXMMS::PlayLoop(InputPlayback *playback)
 {
 	uint32 lLength;
 	//the user might change the number of channels while playing.
@@ -288,9 +303,9 @@ void ModplugXMMS::PlayLoop()
 		if(mStopped)
 			break;
 		
-		produce_audio
+		playback->pass_audio
 		(
-			mPlayed,
+			playback,
 			mFormat,
 			lChannels,
 			mBufSize,
@@ -316,12 +331,12 @@ void ModplugXMMS::PlayLoop()
 
 	mPaused = false;
 	mStopped = true;
-
-	g_thread_exit(NULL);
 }
 
-void ModplugXMMS::PlayFile(const string& aFilename)
+void ModplugXMMS::PlayFile(const string& aFilename, InputPlayback *ipb)
 {
+	int32 aLength=0;
+	char *aModName=NULL;
 	mStopped = true;
 	mPaused = false;
 	
@@ -406,34 +421,16 @@ void ModplugXMMS::PlayFile(const string& aFilename)
 		mArchive->Size()
 	);
 	mPlayed = 0;
-	
-	bool useFilename = mModProps.mUseFilename;
-	
-	if(!useFilename)
-	{
-		strncpy(mModName, mSoundFile->GetTitle(), 100);
-		
-		for(int i = 0; mModName[i] == ' ' || mModName[i] == 0; i++)
-		{
-			if(mModName[i] == 0)
-			{
-				useFilename = true;  //mod name is blank -- use filename
-				break;
-			}
-		}
-	}
-	
-	if(useFilename)
-	{
-		strncpy(mModName, strrchr(aFilename.c_str(), '/') + 1, 100);
-		char* ext = strrchr(mModName, '.');
-		if(ext) *ext = '\0';
-	}
-	
-	mInPlug->set_info
+
+        Tuple* ti = GetSongTuple( aFilename );
+        if ( ti )
+                aModName = format_and_free_ti( ti, &aLength );
+
+	ipb->set_params
 	(
-		mModName,
-		mSoundFile->GetSongTime() * 1000,
+		ipb,
+		aModName,
+		aLength,
 		mSoundFile->GetNumChannels() * 1000,
 		mModProps.mFrequency,
 		mModProps.mChannels
@@ -453,12 +450,9 @@ void ModplugXMMS::PlayFile(const string& aFilename)
 		mModProps.mChannels
 	);
 
-	mDecodeThread = g_thread_create(
-		(GThreadFunc)PlayThread,
-		this,
-		TRUE,
-		NULL
-	);
+	mDecodeThread = g_thread_self();
+	ipb->set_pb_ready(ipb);
+	this->PlayLoop(ipb);
 }
 
 void ModplugXMMS::Stop(void)
@@ -501,151 +495,70 @@ void ModplugXMMS::Seek(float32 aTime)
 
 float32 ModplugXMMS::GetTime(void)
 {
-	if(mStopped)
+	if ((mStopped) || (!mOutPlug))
 		return -1;
 	return (float32)mOutPlug->output_time() / 1000;
 }
 
-void ModplugXMMS::GetSongInfo(const string& aFilename, char*& aTitle, int32& aLength)
+Tuple* ModplugXMMS::GetSongTuple(const string& aFilename)
 {
-	aLength = -1;
-	fstream lTestFile;
-	string lError;
-	bool lDone;
-	
-	lTestFile.open(aFilename.c_str(), ios::in);
-	if(!lTestFile)
-	{
-		lError = "**no such file: ";
-		lError += strrchr(aFilename.c_str(), '/') + 1;
-		aTitle = new char[lError.length() + 1];
-		strcpy(aTitle, lError.c_str());
-		return;
-	}
-	
-	lTestFile.close();
-
-	if(mModProps.mFastinfo)
-	{
-		if(mModProps.mUseFilename)
-		{
-			//Use filename as name
-			aTitle = new char[aFilename.length() + 1];
-			strcpy(aTitle, strrchr(aFilename.c_str(), '/') + 1);
-			*strrchr(aTitle, '.') = '\0';
-			return;
-		}
-		
-		fstream lModFile;
-		string lExt;
-		uint32 lPos;
-		
-		lDone = true;
-
-		// previously ios::nocreate was used (X Standard C++ Library)
-		lModFile.open(aFilename.c_str(), ios::in);
-
-		lPos = aFilename.find_last_of('.');
-		if((int)lPos == 0)
-			return;
-		lExt = aFilename.substr(lPos);
-		for(uint32 i = 0; i < lExt.length(); i++)
-			lExt[i] = tolower(lExt[i]);
-
-		if (lExt == ".mod")
-		{
-			lModFile.read(mModName, 20);
-			mModName[20] = 0;
-		}
-		else if (lExt == ".s3m")
-		{
-			lModFile.read(mModName, 28);
-			mModName[28] = 0;
-		}
-		else if (lExt == ".xm")
-		{
-			lModFile.seekg(17);
-			lModFile.read(mModName, 20);
-			mModName[20] = 0;
-		}
-		else if (lExt == ".it")
-		{
-			lModFile.seekg(4);
-			lModFile.read(mModName, 28);
-			mModName[28] = 0;
-		}
-		else
-			lDone = false;     //fall back to slow info
-
-		lModFile.close();
-
-		if(lDone)
-		{
-			for(int i = 0; mModName[i] != 0; i++)
-			{
-				if(mModName[i] != ' ')
-				{
-					aTitle = new char[strlen(mModName) + 1];
-					strcpy(aTitle, mModName);
-					
-					return;
-				}
-			}
-			
-			//mod name is blank.  Use filename instead.
-			aTitle = new char[aFilename.length() + 1];
-			strcpy(aTitle, strrchr(aFilename.c_str(), '/') + 1);
-			*strrchr(aTitle, '.') = '\0';
-			return;
-		}
-	}
-		
-	Archive* lArchive;
 	CSoundFile* lSoundFile;
-	const char* lTitle;
-
+	Archive* lArchive;
+	const gchar *tmps;
+	
 	//open and mmap the file
-	lArchive = OpenArchive(aFilename);
-	if(lArchive->Size() == 0)
-	{
-		lError = "**bad mod file: ";
-		lError += strrchr(aFilename.c_str(), '/') + 1;
-		aTitle = new char[lError.length() + 1];
-		strcpy(aTitle, lError.c_str());
-		delete lArchive;
-		return;
-	}
+        lArchive = OpenArchive(aFilename);
+        if(lArchive->Size() == 0)
+        {
+                delete lArchive;
+                return NULL;
+        }
 
+	Tuple *ti = aud_tuple_new_from_filename(aFilename.c_str());
 	lSoundFile = new CSoundFile;
 	lSoundFile->Create((uchar*)lArchive->Map(), lArchive->Size());
-
-	if(!mModProps.mUseFilename)
-	{
-		lTitle = lSoundFile->GetTitle();
-		
-		for(int i = 0; lTitle[i] != 0; i++)
-		{
-			if(lTitle[i] != ' ')
-			{
-				aTitle = new char[strlen(lTitle) + 1];
-				strcpy(aTitle, lTitle);
-				goto therest;     //sorry
-			}
-		}
-	}
 	
-	//mod name is blank, or user wants the filename to be used as the title.
-	aTitle = new char[aFilename.length() + 1];
-	strcpy(aTitle, strrchr(aFilename.c_str(), '/') + 1);
-	*strrchr(aTitle, '.') = '\0';
-
-therest:
-	aLength = lSoundFile->GetSongTime() * 1000;                   //It wants milliseconds!?!
-
+	switch(lSoundFile->GetType())
+        {
+	case MOD_TYPE_MOD:	tmps = "ProTracker"; break;
+	case MOD_TYPE_S3M:	tmps = "Scream Tracker 3"; break;
+	case MOD_TYPE_XM:	tmps = "Fast Tracker 2"; break;
+	case MOD_TYPE_IT:	tmps = "Impulse Tracker"; break;
+	case MOD_TYPE_MED:	tmps = "OctaMed"; break;
+	case MOD_TYPE_MTM:	tmps = "MultiTracker Module"; break;
+	case MOD_TYPE_669:	tmps = "669 Composer / UNIS 669"; break;
+	case MOD_TYPE_ULT:	tmps = "Ultra Tracker"; break;
+	case MOD_TYPE_STM:	tmps = "Scream Tracker"; break;
+	case MOD_TYPE_FAR:	tmps = "Farandole"; break;
+	case MOD_TYPE_AMF:	tmps = "ASYLUM Music Format"; break;
+	case MOD_TYPE_AMS:	tmps = "AMS module"; break;
+	case MOD_TYPE_DSM:	tmps = "DSIK Internal Format"; break;
+	case MOD_TYPE_MDL:	tmps = "DigiTracker"; break;
+	case MOD_TYPE_OKT:	tmps = "Oktalyzer"; break;
+	case MOD_TYPE_DMF:	tmps = "Delusion Digital Music Fileformat (X-Tracker)"; break;
+	case MOD_TYPE_PTM:	tmps = "PolyTracker"; break;
+	case MOD_TYPE_DBM:	tmps = "DigiBooster Pro"; break;
+	case MOD_TYPE_MT2:	tmps = "MadTracker 2"; break;
+	case MOD_TYPE_AMF0:	tmps = "AMF0"; break;
+	case MOD_TYPE_PSM:	tmps = "Protracker Studio Module"; break;
+	default:		tmps = "ModPlug unknown"; break;
+	}
+	aud_tuple_associate_string(ti, FIELD_CODEC, NULL, tmps);
+	aud_tuple_associate_string(ti, FIELD_QUALITY, NULL, "sequenced");
+	aud_tuple_associate_int(ti, FIELD_LENGTH, NULL, lSoundFile->GetSongTime() * 1000);
+	
+	gchar *tmps2 = MODPLUG_CONVERT(lSoundFile->GetTitle());
+	// Chop any leading spaces off. They are annoying in the playlist.
+	gchar *tmps3 = tmps2; // Make another pointer so tmps2 can still be free()d
+	while ( *tmps3 == ' ' ) tmps3++ ;
+	aud_tuple_associate_string(ti, FIELD_TITLE, NULL, tmps3);
+	g_free(tmps2);
+	
 	//unload the file
 	lSoundFile->Destroy();
 	delete lSoundFile;
 	delete lArchive;
+	return ti;
 }
 
 void ModplugXMMS::SetInputPlugin(InputPlugin& aInPlugin)
@@ -723,32 +636,32 @@ void ModplugXMMS::SetModProps(const Settings& aModProps)
 	CSoundFile::SetResamplingMode(mModProps.mResamplingMode);
 	mPreampFactor = exp(mModProps.mPreampLevel);
 
-	db = bmp_cfg_db_open();
+	db = aud_cfg_db_open();
 
-	bmp_cfg_db_set_bool(db,"modplug","Surround", mModProps.mSurround);
-        bmp_cfg_db_set_bool(db,"modplug","Oversampling", mModProps.mOversamp);
-        bmp_cfg_db_set_bool(db,"modplug","Megabass", mModProps.mMegabass);
-        bmp_cfg_db_set_bool(db,"modplug","NoiseReduction", mModProps.mNoiseReduction);
-        bmp_cfg_db_set_bool(db,"modplug","VolumeRamp", mModProps.mVolumeRamp);
-        bmp_cfg_db_set_bool(db,"modplug","Reverb", mModProps.mReverb);
-        bmp_cfg_db_set_bool(db,"modplug","FastInfo", mModProps.mFastinfo);
-        bmp_cfg_db_set_bool(db,"modplug","UseFileName", mModProps.mUseFilename);
-        bmp_cfg_db_set_bool(db,"modplug","GrabAmigaMOD", mModProps.mGrabAmigaMOD);
-        bmp_cfg_db_set_bool(db,"modplug","PreAmp", mModProps.mPreamp);
-        bmp_cfg_db_set_float(db,"modplug","PreAmpLevel", mModProps.mPreampLevel);
-        bmp_cfg_db_set_int(db,"modplug", "Channels", mModProps.mChannels);
-        bmp_cfg_db_set_int(db,"modplug", "Bits", mModProps.mBits);
-        bmp_cfg_db_set_int(db,"modplug", "Frequency", mModProps.mFrequency);
-        bmp_cfg_db_set_int(db,"modplug", "ResamplineMode", mModProps.mResamplingMode);
-        bmp_cfg_db_set_int(db,"modplug", "ReverbDepth", mModProps.mReverbDepth);
-        bmp_cfg_db_set_int(db,"modplug", "ReverbDelay", mModProps.mReverbDelay);
-        bmp_cfg_db_set_int(db,"modplug", "BassAmount", mModProps.mBassAmount);
-        bmp_cfg_db_set_int(db,"modplug", "BassRange", mModProps.mBassRange);
-        bmp_cfg_db_set_int(db,"modplug", "SurroundDepth", mModProps.mSurroundDepth);
-        bmp_cfg_db_set_int(db,"modplug", "SurroundDelay", mModProps.mSurroundDelay);
-        bmp_cfg_db_set_int(db,"modplug", "LoopCount", mModProps.mLoopCount);
+	aud_cfg_db_set_bool(db, MODPLUG_CFGID,"Surround", mModProps.mSurround);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"Oversampling", mModProps.mOversamp);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"Megabass", mModProps.mMegabass);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"NoiseReduction", mModProps.mNoiseReduction);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"VolumeRamp", mModProps.mVolumeRamp);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"Reverb", mModProps.mReverb);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"FastInfo", mModProps.mFastinfo);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"UseFileName", mModProps.mUseFilename);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"GrabAmigaMOD", mModProps.mGrabAmigaMOD);
+        aud_cfg_db_set_bool(db, MODPLUG_CFGID,"PreAmp", mModProps.mPreamp);
+        aud_cfg_db_set_float(db, MODPLUG_CFGID,"PreAmpLevel", mModProps.mPreampLevel);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "Channels", mModProps.mChannels);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "Bits", mModProps.mBits);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "Frequency", mModProps.mFrequency);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "ResamplineMode", mModProps.mResamplingMode);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "ReverbDepth", mModProps.mReverbDepth);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "ReverbDelay", mModProps.mReverbDelay);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "BassAmount", mModProps.mBassAmount);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "BassRange", mModProps.mBassRange);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "SurroundDepth", mModProps.mSurroundDepth);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "SurroundDelay", mModProps.mSurroundDelay);
+        aud_cfg_db_set_int(db, MODPLUG_CFGID, "LoopCount", mModProps.mLoopCount);
 
-	bmp_cfg_db_close(db);
+	aud_cfg_db_close(db);
 }
 
 ModplugXMMS gModplugXMMS;
