@@ -28,9 +28,9 @@
  * information.
  */
 
-#ifdef HAVE_CONFIG_H
+/* #define AUD_DEBUG 1 */
+
 #include "config.h"
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,17 +39,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <glib.h>
-#include <pthread.h>
 #include <string.h>
 
-#include <audacious/util.h>
 #include <audacious/plugin.h>
-#include <audacious/titlestring.h>
-#include <audacious/vfs.h>
+#include <audacious/util.h>
+#include <audacious/main.h>
 #include <audacious/output.h>
 #include <audacious/strings.h>
 #include <audacious/i18n.h>
-
 #include <audacious/id3tag.h>
 
 #define  PLUGIN_VERSION "1.4"
@@ -67,62 +64,43 @@ static int  is_our_file (char *filename);
 static void play_file (InputPlayback *playback);
 static void tta_pause (InputPlayback *playback, short paused);
 static void stop (InputPlayback *playback);
-static void seek (InputPlayback *playback, int time);
-static int  get_time (InputPlayback *playback);
+static void mseek (InputPlayback *playback, gulong millisec);
+static void seek (InputPlayback *playback, int sec);
 static void get_song_info (char *filename, char **title, int *length);
 static void file_info (char *filename);
 static void about ();
-static TitleInput *get_song_tuple(char *filename);
-static gchar *extname(const char *filename);
+static Tuple *get_song_tuple(char *filename);
+//static gchar *extname(const char *filename);
 
-static pthread_t decode_thread;
+static GThread *decode_thread = NULL;
 static char sample_buffer[PCM_BUFFER_LENGTH * MAX_BSIZE * MAX_NCH];
 static tta_info info;		// currently playing file info
 static int seek_position = -1;
-static int playing = FALSE;
 static int read_samples = -1;
 
 gchar *tta_fmts[] = { "tta", NULL };
 
-InputPlugin tta_ip = 
+InputPlugin tta_ip =
 {
-	NULL,
-	NULL,
-	NULL,
-	init,
-	about,
-	NULL,
-	is_our_file,
-	NULL,
-	play_file,
-	stop,
-	tta_pause,
-	seek,
-	NULL,
-	get_time,
-	NULL,
-	NULL,
-	cleanup,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	get_song_info,
-	file_info,	
-	NULL,
-	get_song_tuple, // get_song_tuple
-	NULL, // set_song_tuple
-	NULL, // buffer
-	NULL, // vfs
-	tta_fmts,
+	.description = "True Audio Plugin",
+	.init = init,
+	.about = about,
+	.is_our_file = is_our_file,
+	.play_file = play_file,
+	.stop = stop,
+	.pause = tta_pause,
+	.seek = seek,
+	.cleanup = cleanup,
+	.get_song_info = get_song_info,
+	.file_info_box = file_info,
+	.get_song_tuple = get_song_tuple,
+	.vfs_extensions = tta_fmts,
+	.mseek = mseek,
 };
 
-InputPlugin *
-get_iplugin_info (void)
-{
-	tta_ip.description = g_strdup_printf ("True Audio Plugin %s", PLUGIN_VERSION);
-	return &tta_ip;
-}
+InputPlugin *tta_iplist[] = { &tta_ip, NULL };
+
+DECLARE_PLUGIN(tta, NULL, NULL, tta_iplist, NULL, NULL, NULL, NULL, NULL);
 
 size_t
 file_size (char *filename)
@@ -130,11 +108,11 @@ file_size (char *filename)
 	VFSFile *f;
 	size_t size = -1;
 
-	if ((f = vfs_fopen (filename, "r")))
+	if ((f = aud_vfs_fopen (filename, "r")))
 	{
-	    vfs_fseek (f, 0, SEEK_END);
-	    size = vfs_ftell (f);
-	    vfs_fclose (f); 
+	    aud_vfs_fseek (f, 0, SEEK_END);
+	    size = aud_vfs_ftell (f);
+	    aud_vfs_fclose (f);
 	}
 	return size;
 }
@@ -149,100 +127,104 @@ tta_error (int error)
 	switch (error)
 	{
         case OPEN_ERROR:
-	    message = "Can't open file\n";
+	    message = _("Can't open file\n");
 	    break;
         case FORMAT_ERROR:
-	    message = "Not supported file format\n";
+	    message = _("Not supported file format\n");
 	    break;
         case FILE_ERROR:
-	    message = "File is corrupted\n";
+	    message = _("File is corrupted\n");
 	    break;
         case READ_ERROR:
-	    message = "Can't read from file\n";
+	    message = _("Can't read from file\n");
 	    break;
         case MEMORY_ERROR:
-	    message = "Insufficient memory available\n";
+	    message = _("Insufficient memory available\n");
 	    break;
         case OUTPUT_ERROR:
-	    message = "Output plugin error\n";
+	    message = _("Output plugin error\n");
 	    break;
 	default:
-	    message = "Unknown error\n";
+	    message = _("Unknown error\n");
 	    break;
 	}
 
-	xmms_show_message ("TTA Decoder Error", message,
-	    "Ok", FALSE, NULL, NULL);
+	audacious_info_dialog (_("TTA Decoder Error"), message,
+	    _("Ok"), FALSE, NULL, NULL);
 
 	gtk_signal_connect(GTK_OBJECT(errorbox), "destroy",
     	    G_CALLBACK(gtk_widget_destroyed), &errorbox);
 }
 
 static gchar *
-get_song_title(TitleInput *tuple)
+get_song_title(Tuple *tuple)
 {
-	return xmms_get_titlestring(xmms_get_gentitle_format(), tuple);
+	return aud_tuple_formatter_make_title_string(tuple, aud_get_gentitle_format());
 }
 
 static void
 get_song_info (char *filename, char **title, int *length)
 {
-	TitleInput *tuple;
+	Tuple *tuple;
 
 	*length = -1;
 	*title = NULL;
 
 	if ((tuple = get_song_tuple(filename)) != NULL) {
-    	    *length = tuple->length;
+    	    *length = aud_tuple_get_int(tuple, FIELD_LENGTH, NULL);
     	    *title = get_song_title(tuple);
 	}
 
-	bmp_title_input_free(tuple);
+	aud_tuple_free(tuple);
 }
 
 static void *
-play_loop (void *arg)
+play_loop (InputPlayback *playback)
 {
-	InputPlayback *playback = arg;
 	int  bufsize = PCM_BUFFER_LENGTH  * info.BSIZE * info.NCH;
 
 	////////////////////////////////////////
 	// decode PCM_BUFFER_LENGTH samples
 	// into the current PCM buffer position
 
-	while (playing)
+	while (playback->playing)
 	{
-	    while ((read_samples = get_samples (sample_buffer)) > 0)
+	    while ((read_samples = get_samples ((unsigned char *)sample_buffer)) > 0)
 	    {
 
-		while ((playback->output->buffer_free () < bufsize)
-		    && seek_position == -1)
-		{
-		    if (!playing)
-			goto DONE;
-		    xmms_usleep (10000);
-		}
-
-		if (seek_position == -1)
-		{
-		    produce_audio(playback->output->written_time(),
-			      ((info.BPS == 8) ? FMT_U8 : FMT_S16_LE),
-			      info.NCH,
-			      read_samples * info.NCH * info.BSIZE,
-			      sample_buffer,
-			      NULL);
-		}
-		else
-		{
-		    set_position (seek_position);
-		    playback->output->flush (seek_position * SEEK_STEP);
-		    seek_position = -1;
-		}
+            while ((playback->output->buffer_free () < bufsize)
+                   && seek_position == -1)
+            {
+                if (!playback->playing)
+                    goto DONE;
+                g_usleep (10000);
+            }
+            if (seek_position == -1)
+            {
+                playback->pass_audio(playback,
+                              ((info.BPS == 8) ? FMT_U8 : FMT_S16_LE),
+                              info.NCH,
+                              read_samples * info.NCH * info.BSIZE,
+                              sample_buffer,
+                              &playback->playing);
+            }
+            else
+            {
+                set_position (seek_position);
+                playback->output->flush (seek_position * SEEK_STEP);
+                seek_position = -1;
+            }
+            if(!playback->playing)
+                goto DONE;
 	    }
 
 	    playback->output->buffer_free ();
 	    playback->output->buffer_free ();
-	    xmms_usleep(10000);
+	    while (playback->output->buffer_playing()) {
+		    g_usleep(10000);
+		    if(!playback->playing)
+			    goto DONE;
+	    }
 	}
 
 DONE:
@@ -254,7 +236,7 @@ DONE:
 	// close currently playing file
 	close_tta_file (&info);
 
-	pthread_exit (NULL);
+	return NULL;
 }
 
 static void
@@ -272,16 +254,21 @@ static void
 about ()
 {
 	static GtkWidget *aboutbox;
+        gchar *about_text;
+
 	if (aboutbox != NULL) return;
 
-	aboutbox = xmms_show_message(
-	    "About True Audio Plugin",
-	    "TTA input plugin" PLUGIN_VERSION "for BMP\n"
-	    "Copyright (c) 2004 True Audio Software\n"
-	    PROJECT_URL, "Ok", FALSE, NULL, NULL);
+	about_text = g_strjoin("", _("TTA input plugin "), PLUGIN_VERSION,
+				   _(" for BMP\n"
+		        	   "Copyright (c) 2004 True Audio Software\n"), PROJECT_URL, NULL);
+
+	aboutbox = audacious_info_dialog(_("About True Audio Plugin"),
+				     about_text,
+				     _("Ok"), FALSE, NULL, NULL);
 
 	gtk_signal_connect(GTK_OBJECT(aboutbox), "destroy",
-    	    G_CALLBACK(gtk_widget_destroyed), &aboutbox);
+			   G_CALLBACK(gtk_widget_destroyed), &aboutbox);
+	g_free(about_text);
 }
 
 static GtkWidget *window = NULL;
@@ -297,6 +284,7 @@ file_info (char *filename)
 	tta_info ttainfo;
 	char *title;
 	gchar *utf_filename = NULL;
+	gchar *realfn = NULL;
 
 	if (!window) {
 	    GtkWidget *vbox, *hbox, *left_vbox, *table;
@@ -307,13 +295,13 @@ file_info (char *filename)
 	    gtk_signal_connect(GTK_OBJECT(window), "destroy",
 		G_CALLBACK(gtk_widget_destroyed), &window);
 	    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
-    
+
 	    vbox = gtk_vbox_new(FALSE, 10);
 	    gtk_container_add(GTK_CONTAINER(window), vbox);
 
 	    filename_hbox = gtk_hbox_new(FALSE, 5);
 	    gtk_box_pack_start(GTK_BOX(vbox), filename_hbox, FALSE, TRUE, 0);
-	    label = gtk_label_new("Filename:");
+	    label = gtk_label_new(_("Filename:"));
 	    gtk_box_pack_start(GTK_BOX(filename_hbox), label, FALSE, TRUE, 0);
 
 	    filename_entry = gtk_entry_new_with_max_length(1024);
@@ -325,14 +313,14 @@ file_info (char *filename)
 	    left_vbox = gtk_vbox_new(FALSE, 10);
 	    gtk_box_pack_start(GTK_BOX(hbox), left_vbox, FALSE, FALSE, 0);
 
-	    info_frame = gtk_frame_new("ID3 Tag:");
+	    info_frame = gtk_frame_new(_("ID3 Tag:"));
 	    gtk_box_pack_start(GTK_BOX(left_vbox), info_frame, FALSE, FALSE, 0);
 
 	    table = gtk_table_new(5, 5, FALSE);
 	    gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 	    gtk_container_add(GTK_CONTAINER(info_frame), table);
 
-	    label = gtk_label_new("Title:");
+	    label = gtk_label_new(_("Title:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 5, 5);
 
@@ -342,7 +330,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    label = gtk_label_new("Artist:");
+	    label = gtk_label_new(_("Artist:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -353,7 +341,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    label = gtk_label_new("Album:");
+	    label = gtk_label_new(_("Album:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -364,7 +352,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    label = gtk_label_new("Comment:");
+	    label = gtk_label_new(_("Comment:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -375,7 +363,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    label = gtk_label_new("Year:");
+	    label = gtk_label_new(_("Year:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -387,7 +375,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
- 	    label = gtk_label_new("Track number:");
+ 	    label = gtk_label_new(_("Track number:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 4, 5,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -399,7 +387,7 @@ file_info (char *filename)
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    label = gtk_label_new("Genre:");
+	    label = gtk_label_new(_("Genre:"));
 	    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
 		GTK_FILL, GTK_FILL, 5, 5);
@@ -411,7 +399,7 @@ file_info (char *filename)
         	GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 
-	    button_ok = gtk_button_new_with_label("Ok");
+	    button_ok = gtk_button_new_with_label(_("Ok"));
 	    gtk_signal_connect_object(GTK_OBJECT(button_ok), "clicked",
 		G_CALLBACK(gtk_widget_destroy), G_OBJECT(window));
 	    GTK_WIDGET_SET_FLAGS(button_ok, GTK_CAN_DEFAULT);
@@ -419,12 +407,13 @@ file_info (char *filename)
 
 	    gtk_widget_show_all (window);
 	}
-    
-	utf_filename = str_to_utf8(filename);
+	realfn = g_filename_from_uri(filename, NULL, NULL);
+	utf_filename = aud_str_to_utf8(realfn ? realfn : filename);
+	g_free(realfn); realfn = NULL;
 	title = g_strdup_printf(_("File Info - %s"), g_basename(utf_filename));
 	gtk_window_set_title(GTK_WINDOW(window), title);
 	g_free(title);
-    
+
 	gtk_entry_set_text(GTK_ENTRY(filename_entry), utf_filename);
 	gtk_editable_set_position(GTK_EDITABLE(filename_entry), -1);
 
@@ -436,13 +425,13 @@ file_info (char *filename)
 
 	if (open_tta_file (filename, &ttainfo, 0) >= 0)
 	{
-	    gtk_entry_set_text(GTK_ENTRY(title_entry), ttainfo.ID3.title);
-	    gtk_entry_set_text(GTK_ENTRY(artist_entry), ttainfo.ID3.artist);
-	    gtk_entry_set_text(GTK_ENTRY(album_entry), ttainfo.ID3.album);
-	    gtk_entry_set_text(GTK_ENTRY(year_entry), ttainfo.ID3.year);
-	    gtk_entry_set_text(GTK_ENTRY(tracknum_entry), ttainfo.ID3.track);
-	    gtk_entry_set_text(GTK_ENTRY(comment_entry), ttainfo.ID3.comment);
-	    gtk_entry_set_text(GTK_ENTRY(genre_entry), ttainfo.ID3.genre);
+	    gtk_entry_set_text(GTK_ENTRY(title_entry), (gchar *)ttainfo.ID3.title);
+	    gtk_entry_set_text(GTK_ENTRY(artist_entry), (gchar *)ttainfo.ID3.artist);
+	    gtk_entry_set_text(GTK_ENTRY(album_entry), (gchar *)ttainfo.ID3.album);
+	    gtk_entry_set_text(GTK_ENTRY(year_entry), (gchar *)ttainfo.ID3.year);
+	    gtk_entry_set_text(GTK_ENTRY(tracknum_entry), (gchar *)ttainfo.ID3.track);
+	    gtk_entry_set_text(GTK_ENTRY(comment_entry), (gchar *)ttainfo.ID3.comment);
+	    gtk_entry_set_text(GTK_ENTRY(genre_entry), (gchar *)ttainfo.ID3.genre);
 	}
 	close_tta_file (&ttainfo);
 
@@ -463,16 +452,14 @@ is_our_file (char *filename)
 static void
 play_file (InputPlayback *playback)
 {
-	char *filename = playback->filename;
-	char *title;
+	gchar *filename = playback->filename;
+	char *title = NULL;
 	int datasize, origsize, bitrate;
-	TitleInput *tuple = NULL;
-
-	playing = FALSE;
+	Tuple *tuple = NULL;
 
 	////////////////////////////////////////
 	// open TTA file
-	if (open_tta_file (filename, &info, 0) < 0)
+	if (open_tta_file (filename, &info, 0) > 0)
 	{
 	    tta_error (info.STATE);
 	    close_tta_file (&info);
@@ -498,7 +485,7 @@ play_file (InputPlayback *playback)
 
 	tuple = get_song_tuple(filename);
 	title = get_song_title(tuple);
-	bmp_title_input_free(tuple);
+	aud_tuple_free(tuple);
 
 	datasize = file_size(filename) - info.DATAPOS;
 	origsize = info.DATALENGTH * info.BSIZE * info.NCH;
@@ -506,17 +493,18 @@ play_file (InputPlayback *playback)
 	bitrate  = (int) ((float) datasize / origsize *
 	        (info.SAMPLERATE * info.NCH * info.BPS));
 
-	tta_ip.set_info (title, 1000 * info.LENGTH, bitrate, info.SAMPLERATE, info.NCH);
-    
+	playback->set_params(playback, title, 1000 * info.LENGTH, bitrate, info.SAMPLERATE, info.NCH);
+
 	if (title)
 	    g_free (title);
 
-	playing = TRUE;
 	playback->playing = 1;
 	seek_position = -1;
 	read_samples = -1;
 
-	pthread_create (&decode_thread, NULL, play_loop, playback);
+	decode_thread = g_thread_self();
+	playback->set_pb_ready(playback);
+	play_loop(playback);
 }
 
 static void
@@ -528,85 +516,86 @@ tta_pause (InputPlayback *playback, short paused)
 static void
 stop (InputPlayback *playback)
 {
-	if (playing)
+	if (playback->playing)
 	{
-	    playing = FALSE;
 	    playback->playing = 0;
-	    pthread_join (decode_thread, NULL);
+	    g_thread_join(decode_thread);
+	    decode_thread = NULL;
 	    playback->output->close_audio ();
 	    close_tta_file (&info);
-	    read_samples = 0;
+	    read_samples = -1;
 	}
 }
 
 static void
-seek (InputPlayback *data, int time)
+mseek (InputPlayback *data, gulong millisec)
 {
-	if (playing)
+	if (data->playing)
 	{
-	    seek_position = 1000 * time / SEEK_STEP;
+	    seek_position = (int)(millisec / SEEK_STEP);
 
 	    while (seek_position != -1)
-		xmms_usleep (10000);
+		g_usleep (10000);
 	}
 }
 
-static int
-get_time (InputPlayback *playback)
+static void
+seek (InputPlayback *data, int sec)
 {
-	if (playing && (read_samples || playback->output->buffer_playing()))
-    	    return playback->output->output_time();
-
-	return -1;
+    gulong millisec = 1000 * sec;
+    mseek(data, millisec);
 }
 
-static TitleInput *
+static Tuple *
 get_song_tuple(char *filename)
 {
-	TitleInput *tuple = NULL;
+	Tuple *tuple = NULL;
 	tta_info *ttainfo;
 	VFSFile *file;
 
 	ttainfo = g_malloc0(sizeof(tta_info));
 
-	if((file = vfs_fopen(filename, "rb")) != NULL) {
+	if((file = aud_vfs_fopen(filename, "rb")) != NULL) {
 		if(open_tta_file(filename, ttainfo, 0) >= 0) {
-			tuple = bmp_title_input_new();
+			tuple = aud_tuple_new_from_filename(filename);
 
-			tuple->file_name = g_path_get_basename(filename);
-			tuple->file_path = g_path_get_dirname(filename);
-			tuple->file_ext = extname(filename);
-			tuple->length = ttainfo->LENGTH * 1000;
+			aud_tuple_associate_string(tuple, FIELD_CODEC, NULL, "True Audio (TTA)");
+			aud_tuple_associate_string(tuple, FIELD_QUALITY, NULL, "lossless");
 
 			if (ttainfo->ID3.id3has) {
-				if(ttainfo->ID3.artist && strlen(ttainfo->ID3.artist))
-					tuple->performer = g_strdup(ttainfo->ID3.artist);
+				if (ttainfo->ID3.artist)
+					aud_tuple_associate_string(tuple, FIELD_ARTIST, NULL, (gchar *) ttainfo->ID3.artist);
 
-				if(ttainfo->ID3.album && strlen(ttainfo->ID3.album))
-					tuple->album_name = g_strdup(ttainfo->ID3.album);
+				if (ttainfo->ID3.album)
+					aud_tuple_associate_string(tuple, FIELD_ALBUM, NULL, (gchar *) ttainfo->ID3.album);
 
-				if(ttainfo->ID3.title && strlen(ttainfo->ID3.title))
-					tuple->track_name = g_strdup(ttainfo->ID3.title);
+				if (ttainfo->ID3.title)
+					aud_tuple_associate_string(tuple, FIELD_TITLE, NULL, (gchar *) ttainfo->ID3.title);
 
-				if(ttainfo->ID3.year && strlen(ttainfo->ID3.year))
-					tuple->year = atoi(ttainfo->ID3.year);
+				if (ttainfo->ID3.year)
+					aud_tuple_associate_int(tuple, FIELD_YEAR, NULL, atoi((char *)ttainfo->ID3.year));
 
-				if(ttainfo->ID3.track && strlen(ttainfo->ID3.track))
-					tuple->track_number = atoi(ttainfo->ID3.track);
+				if(ttainfo->ID3.track)
+					aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, atoi((char *)ttainfo->ID3.track));
 
-				if(ttainfo->ID3.genre && strlen(ttainfo->ID3.genre))
-					tuple->genre = g_strdup(ttainfo->ID3.genre);
+				if(ttainfo->ID3.genre)
+					aud_tuple_associate_string(tuple, FIELD_GENRE, NULL, (gchar *) ttainfo->ID3.genre);
+
+				if(ttainfo->ID3.comment)
+					aud_tuple_associate_string(tuple, FIELD_COMMENT, NULL, (gchar *) ttainfo->ID3.comment);
+				if(ttainfo->LENGTH)
+					aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, 1000 * ttainfo->LENGTH);
 
-				if(ttainfo->ID3.comment && strlen(ttainfo->ID3.comment))
-					tuple->comment = g_strdup(ttainfo->ID3.comment);
 			}
 			close_tta_file (ttainfo);
 		}
-		vfs_fclose(file);
+		aud_vfs_fclose(file);
 	}
+        g_free(ttainfo);
 	return tuple;
 }
 
+#if 0
 static gchar *
 extname(const char *filename)
 {
@@ -617,6 +606,7 @@ extname(const char *filename)
 
 	return ext;
 }
+#endif
 
 /* return length in letters */
 size_t tta_ucs4len(id3_ucs4_t *ucs)
@@ -708,18 +698,14 @@ id3_ucs4_t *tta_parse_genre(const id3_ucs4_t *string)
                 tp++;
             }
             if(is_num) {
-#ifdef DEBUG
-                printf("is_num!\n");
-#endif
+                AUDDBG("is_num!\n");
                 tmp = g_malloc0(BYTES(end - ptr + 1));
                 memcpy(tmp, ptr, BYTES(end - ptr));
                 *(tmp + (end - ptr)) = 0; //terminate
                 ptr += end - ptr;
 
                 genre = (id3_ucs4_t *)id3_genre_name((const id3_ucs4_t *)tmp);
-#ifdef DEBUG
-                printf("genre length = %d\n", tta_ucs4len(genre));
-#endif
+                AUDDBG("genre length = %d\n", tta_ucs4len(genre));
                 g_free(tmp);
                 tmp = NULL;
 
@@ -731,10 +717,8 @@ id3_ucs4_t *tta_parse_genre(const id3_ucs4_t *string)
                 *(ret + ret_len) = 0; //terminate
             }
             else { // plain text
-#ifdef DEBUG
-                printf("plain!\n");
-                printf("ret_len = %d\n", ret_len);
-#endif
+                AUDDBG("plain!\n");
+                AUDDBG("ret_len = %d\n", ret_len);
                 memcpy(ret + BYTES(ret_len), ptr, BYTES(end - ptr));
                 ret_len = ret_len + (end - ptr);
                 *(ret + ret_len) = 0; //terminate
@@ -761,7 +745,7 @@ gchar *tta_input_id3_get_string(struct id3_tag * tag, char *frame_name)
     if (!frame)
         return NULL;
 
-    if (frame_name == ID3_FRAME_COMMENT)
+    if (!strcmp(frame_name, ID3_FRAME_COMMENT))
         field = id3_frame_field(frame, 3);
     else
         field = id3_frame_field(frame, 1);
@@ -769,7 +753,7 @@ gchar *tta_input_id3_get_string(struct id3_tag * tag, char *frame_name)
     if (!field)
         return NULL;
 
-    if (frame_name == ID3_FRAME_COMMENT)
+    if (!strcmp(frame_name, ID3_FRAME_COMMENT))
         string_const = id3_field_getfullstring(field);
     else
         string_const = id3_field_getstrings(field, 0);
@@ -779,7 +763,7 @@ gchar *tta_input_id3_get_string(struct id3_tag * tag, char *frame_name)
 
     string = tta_ucs4dup((id3_ucs4_t *)string_const);
 
-    if (frame_name == ID3_FRAME_GENRE) {
+    if (!strcmp(frame_name, ID3_FRAME_GENRE)) {
         id3_ucs4_t *string2 = NULL;
         string2 = tta_parse_genre(string);
         g_free((void *)string);
@@ -799,19 +783,17 @@ gchar *tta_input_id3_get_string(struct id3_tag * tag, char *frame_name)
 #ifdef DEBUG
         g_message("aud-tta: flagutf!\n");
 #endif
-        rtn = id3_ucs4_utf8duplicate(string);
+        rtn = (gchar *)id3_ucs4_utf8duplicate(string);
     }
     else {
-        rtn = id3_ucs4_latin1duplicate(string);
-        rtn2 = str_to_utf8(rtn);
+        rtn = (gchar *)id3_ucs4_latin1duplicate(string);
+        rtn2 = aud_str_to_utf8(rtn);
         free(rtn);
         rtn = rtn2;
     }
     g_free(string);
     string = NULL;
-#ifdef DEBUG
-    g_print("string = %s\n", rtn);
-#endif    
+    AUDDBG("string = %s\n", rtn);
 
     return rtn;
 }
@@ -833,38 +815,38 @@ int get_id3_tags (const char *filename, tta_info *ttainfo) {
 			id3v2_size = tag->paddedsize;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_ARTIST);
-			if(str) strncpy(ttainfo->ID3.artist, str, MAX_LINE);
+			if(str) strncpy((char *)ttainfo->ID3.artist, str, MAX_LINE);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_ALBUM);
-			if(str) strncpy(ttainfo->ID3.album, str, MAX_LINE);
+			if(str) strncpy((char *)ttainfo->ID3.album, str, MAX_LINE);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_TITLE);
-			if(str) strncpy(ttainfo->ID3.title, str, MAX_LINE);
+			if(str) strncpy((char *)ttainfo->ID3.title, str, MAX_LINE);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_YEAR);
 			if(!str) str = tta_input_id3_get_string (tag, "TYER");
-			if(str) strncpy(ttainfo->ID3.year, str, MAX_YEAR);
+			if(str) strncpy((char *)ttainfo->ID3.year, str, MAX_YEAR);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_TRACK);
-			if(str) strncpy(ttainfo->ID3.track, str, MAX_TRACK);
+			if(str) strncpy((char *)ttainfo->ID3.track, str, MAX_TRACK);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_GENRE);
-			if(str) strncpy(ttainfo->ID3.genre, str, MAX_GENRE);
+			if(str) strncpy((char *)ttainfo->ID3.genre, str, MAX_GENRE);
 			free(str);
 			str = NULL;
 
 			str = tta_input_id3_get_string (tag, ID3_FRAME_COMMENT);
-			if(str) strncpy(ttainfo->ID3.comment, str, MAX_LINE);
+			if(str) strncpy((char *)ttainfo->ID3.comment, str, MAX_LINE);
 			free(str);
 			str = NULL;
 		}
