@@ -21,8 +21,9 @@
 
 #include <audacious/output.h>
 #include <audacious/util.h>
-#include <audacious/titlestring.h>
+#include <audacious/configdb.h>
 #include <audacious/vfs.h>
+#include <audacious/main.h>
 #include <audacious/strings.h>
 #include <audacious/i18n.h>
 
@@ -56,12 +57,25 @@ ayemu_vtx_t vtx;
 
 static gchar *vtx_fmts[] = { "vtx", NULL };
 
+void
+vtx_init(void)
+{
+        ConfigDb *db;
+        db = aud_cfg_db_open();
+
+        aud_cfg_db_get_int(db, NULL, "src_rate", &freq);
+        if (freq < 4000 || freq > 192000)
+                freq = 44100;
+
+        aud_cfg_db_close(db);
+}
+
 int
 vtx_is_our_fd (char *filename, VFSFile *fp)
 {
   char buf[2];
     
-  vfs_fread (buf, 2, 1, fp);
+  aud_vfs_fread (buf, 2, 1, fp);
   return (!strncasecmp (buf, "ay", 2) || !strncasecmp (buf, "ym", 2));
 }
 
@@ -71,43 +85,44 @@ vtx_is_our_file (char *filename)
   gboolean ret;
   VFSFile *fp;
 
-  fp = vfs_fopen(filename, "rb");    
+  fp = aud_vfs_fopen(filename, "rb");    
   ret = vtx_is_our_fd(filename, fp);
-  vfs_fclose(fp);
+  aud_vfs_fclose(fp);
 
   return ret;
 }
 
-TitleInput *
+Tuple *
 vtx_get_song_tuple_from_vtx(const gchar *filename, ayemu_vtx_t *in)
 {
-  TitleInput *out = bmp_title_input_new();
-  gchar *string;
+  Tuple *out = aud_tuple_new_from_filename(filename);
 
-  out->performer = g_strdup(in->hdr.author);
-  out->track_name = g_strdup(in->hdr.title);
+  aud_tuple_associate_string(out, FIELD_ARTIST, NULL, in->hdr.author);
+  aud_tuple_associate_string(out, FIELD_TITLE, NULL, in->hdr.title);
 
-  out->file_name = g_strdup(g_basename(filename));
-  out->file_path = g_path_get_dirname(filename);
-  if ((string = strrchr(out->file_name, '.')))
-    {
-      out->file_ext = string + 1;
-      *string = '\0';
-    }
+  aud_tuple_associate_int(out, FIELD_LENGTH, NULL, in->hdr.regdata_size / 14 * 1000 / 50);
 
-  out->length = in->hdr.regdata_size / 14 * 1000 / 50;
+  aud_tuple_associate_string(out, FIELD_GENRE, NULL, (in->hdr.chiptype == AYEMU_AY)? "AY chiptunes" : "YM chiptunes");
+  aud_tuple_associate_string(out, FIELD_ALBUM, NULL, in->hdr.from);
+  aud_tuple_associate_string(out, -1, "game", in->hdr.from);
+
+  aud_tuple_associate_string(out, FIELD_QUALITY, NULL, "sequenced");
+  aud_tuple_associate_string(out, FIELD_CODEC, NULL, in->hdr.tracker);
+  aud_tuple_associate_string(out, -1, "tracker", in->hdr.tracker);
+
+  aud_tuple_associate_int(out, FIELD_YEAR, NULL, in->hdr.year);
 
   return out;
 }
 
-TitleInput *
+Tuple *
 vtx_get_song_tuple(gchar *filename)
 {
   ayemu_vtx_t tmp;
 
   if (ayemu_vtx_open (&tmp, filename))
     {
-      TitleInput *ti = vtx_get_song_tuple_from_vtx(filename, &tmp);
+      Tuple *ti = vtx_get_song_tuple_from_vtx(filename, &tmp);
       ayemu_vtx_free(&tmp);
       return ti;
     }
@@ -143,7 +158,7 @@ play_loop (gpointer args)
 	  }
 	else
 	  {			/* get next AY register frame */
-	    if (ayemu_vtx_get_next_frame (&vtx, regs) == 0)
+	    if (ayemu_vtx_get_next_frame (&vtx, (char *)regs) == 0)
 	      {
 		playback->eof = TRUE;
 		donow = need;
@@ -162,13 +177,16 @@ play_loop (gpointer args)
 	g_usleep(10000);
 
       if (playback->playing && seek_to == -1)
-        produce_audio(playback->output->written_time (), FMT_S16_NE,
+        playback->pass_audio(playback, FMT_S16_NE,
   			  chans , SNDBUFSIZE, sndbuf, &playback->playing);
     
       if (playback->eof)
 	{
 	  playback->output->buffer_free ();
 	  playback->output->buffer_free ();
+	  while (playback->output->buffer_playing())
+	    g_usleep(10000);
+	  playback->playing = 0;
 	}
     
       /* jump to time in seek_to (in seconds) */
@@ -179,10 +197,7 @@ play_loop (gpointer args)
 	  seek_to = -1;
 	}
     }
-
-  /* close sound and release vtx file must be done in vtx_stop() */
-  g_thread_exit (NULL);
-
+  ayemu_vtx_free (&vtx);
   return NULL;
 }
 
@@ -190,7 +205,7 @@ void vtx_play_file (InputPlayback *playback)
 {
   gchar *filename = playback->filename;
   gchar *buf;
-  TitleInput *ti;
+  Tuple *ti;
 
   memset (&ay, 0, sizeof(ay));
 
@@ -218,18 +233,19 @@ void vtx_play_file (InputPlayback *playback)
       seek_to = -1;
 
       ti = vtx_get_song_tuple_from_vtx(playback->filename, &vtx);
+      buf = aud_tuple_formatter_make_title_string(ti, aud_get_gentitle_format());
 
-      buf = xmms_get_titlestring(xmms_get_gentitle_format(), ti);
-
-      vtx_ip.set_info (buf, vtx.hdr.regdata_size / 14 * 1000 / 50,
+      playback->set_params (playback, buf, vtx.hdr.regdata_size / 14 * 1000 / 50,
  	  	       14 * 50 * 8, freq, bits / 8);
 
       g_free (buf);
 
-      bmp_title_input_free(ti);
+      aud_tuple_free(ti);
 
       playback->playing = TRUE;
-      play_thread = g_thread_create (play_loop, playback, TRUE, NULL);
+      play_thread = g_thread_self();
+      playback->set_pb_ready(playback);
+      play_loop(playback);
     }
 }
 
@@ -279,52 +295,33 @@ vtx_get_song_info (char *filename, char **title, int *length)
   (*title) = NULL;
 
   if (ayemu_vtx_open (&tmp, filename)) {
-    TitleInput *ti = vtx_get_song_tuple_from_vtx(filename, &tmp);
+    Tuple *ti = vtx_get_song_tuple_from_vtx(filename, &tmp);
 
-    *title = xmms_get_titlestring(xmms_get_gentitle_format(), ti);
-    *length = ti->length;
+    *title = aud_tuple_formatter_process_string(ti, aud_get_gentitle_format());
+    *length = aud_tuple_get_int(ti, FIELD_LENGTH, NULL);
 
     ayemu_vtx_free (&tmp);
-    bmp_title_input_free(ti);
+    aud_tuple_free(ti);
   }
 }
 
 InputPlugin vtx_ip = {
-	NULL,			/* FILLED BY XMMS */
-	NULL,			/* FILLED BY XMMS */
-	"VTX Plugin",		/* Plugin description */
-	NULL,			/* Initialization */
-	vtx_about,		/* Show aboutbox */
-	vtx_config,		/* Show/edit configuration */
-	vtx_is_our_file,	/* Check file, return 1 if the plugin can handle this file */
-	NULL,			/* Scan directory */
-	vtx_play_file,		/* Play given file */
-	vtx_stop,		/* Stop playing */
-	vtx_pause,		/* Pause playing */
-	vtx_seek,		/* Seek time */
-	NULL,			/* Set equalizer */
-	NULL,			/* Get playing time (obsoleted by InputPlayback API) */
-	NULL,			/* Get volume */
-	NULL,			/* Set volume */
-	NULL,			/* Cleanup */
-	NULL,			/* OBSOLETE! */
-	NULL,			/* Send data to Visualization plugin */
-	NULL, NULL,		/* FILLED BY XMMS */
-	vtx_get_song_info,	/* Get song title and length */
-	vtx_file_info,		/* Show file-information dialog */
-	NULL,			/* FILLED BY XMMS */
-	vtx_get_song_tuple,	/* Tuple */
-	NULL,			/* Tuple */
-	NULL,			/* Buffer */
-	vtx_is_our_fd,		/* VFS */
-	vtx_fmts		/* ext assist */
+	.description = "VTX Audio Plugin",	/* Plugin description */
+	.init = vtx_init,		/* Initialization */
+	.about = vtx_about,		/* Show aboutbox */
+	.configure = vtx_config,		/* Show/edit configuration */
+	.is_our_file = vtx_is_our_file,	/* Check file, return 1 if the plugin can handle this file */
+	.play_file = vtx_play_file,		/* Play given file */
+	.stop = vtx_stop,		/* Stop playing */
+	.pause = vtx_pause,		/* Pause playing */
+	.seek = vtx_seek,		/* Seek time */
+	.get_song_info = vtx_get_song_info,	/* Get song title and length */
+	.file_info_box = vtx_file_info,		/* Show file-information dialog */
+	.get_song_tuple = vtx_get_song_tuple,	/* Tuple */
+	.is_our_file_from_vfs = vtx_is_our_fd,		/* VFS */
+	.vfs_extensions = vtx_fmts		/* ext assist */
 };
 
-/* called from xmms for plug */
-InputPlugin *
-get_iplugin_info (void)
-{
-  vtx_ip.description = g_strdup (_("VTX Plugin"));
-  return &vtx_ip;
-}
+InputPlugin *vtx_iplist[] = { &vtx_ip, NULL };
 
+SIMPLE_INPUT_PLUGIN(vtx, vtx_iplist);
